import asyncio
import time
import logging
import traceback
import ujson
from typing import Optional, Union
from collections import defaultdict, deque

from .base import BaseExecutor

logger = logging.getLogger('app')

# Try to use uvloop for better performance
try:
    import uvloop
    uvloop.install()
    logger.info("Using uvloop for better performance")
except ImportError:
    pass


class AsyncioExecutor(BaseExecutor):
    """High-performance asyncio executor"""
    
    def __init__(self, event_queue, app, concurrency=100):
        super().__init__(event_queue, app, concurrency)
        
        # Caching for pending count
        self.pending_cache = {}
        self.pending_cache_expire = 0
        
        # 高性能模式默认设置
        self.ack_buffer_size = 500  # 高性能批量大小
        self.max_ack_buffer_size = 2000  # 最大批量大小
        self.status_batch_size = 1000  # 高性能状态更新批量
        self.data_batch_size = 1000   # 高性能数据更新批量
        
        # 性能优化2: 预分配内存避免频繁扩容
        self.pending_acks = []
        self.status_updates = []  
        self.data_updates = []
        self.task_info_updates = {}  # 新增：使用字典存储每个任务的Hash更新
        
        # 预分配初始容量
        self.pending_acks.extend([None] * self.ack_buffer_size)
        self.pending_acks.clear()
        
        # 添加前缀
        self.prefix = self.app.ep.redis_prefix or 'jettask'
        
        # 高性能刷新策略
        self.last_flush_time = time.time()
        self.max_flush_interval = 0.05  # 50ms最大刷新间隔
        self.min_flush_interval = 0.005  # 5ms最小刷新间隔
        
        # 性能优化4: 预编译常量和缓存
        self._status_prefix = self.app._status_prefix
        self._result_prefix = self.app._result_prefix
        self._prefixed_queue_cache = {}  # 缓存队列名称
        
        # 默认启用高性能模式
        self._stats_lock = asyncio.Lock()
        self._high_performance_mode = True  # 始终启用高性能模式
        
    def _get_prefixed_queue_cached(self, queue: str) -> str:
        """缓存队列名称以避免重复字符串拼接"""
        if queue not in self._prefixed_queue_cache:
            self._prefixed_queue_cache[queue] = self.app.ep.get_prefixed_queue_name(queue)
        return self._prefixed_queue_cache[queue]
        
    async def get_pending_count_cached(self, queue: str) -> int:
        """Get cached pending count"""
        current_time = time.time()
        
        if (current_time - self.pending_cache_expire > 30 or  # 优化：延长缓存时间
            queue not in self.pending_cache):
            try:
                pending_info = await self.app.ep.async_redis_client.xpending(queue, queue)
                self.pending_cache[queue] = pending_info.get("pending", 0)
                self.pending_cache_expire = current_time
            except Exception:
                self.pending_cache[queue] = 0
                
        return self.pending_cache.get(queue, 0)
    
    async def _quick_ack(self, queue: str, event_id: str):
        """Quick ACK with adaptive batching and smart flushing"""
        self.pending_acks.append((queue, event_id))
        current_time = time.time()
        
        # 高性能刷新策略：更激进的批量处理
        should_flush = (
            len(self.pending_acks) >= self.ack_buffer_size or  # 达到批量大小
            (len(self.pending_acks) >= 50 and  # 或有50个且超时
             current_time - self.last_flush_time >= self.max_flush_interval) or
            len(self.pending_acks) >= self.max_ack_buffer_size * 0.1  # 达到最大缓冲区10%
        )
        
        if should_flush:
            await self._flush_acks()
    
    async def _flush_acks(self):
        """Batch execute ACKs - optimized"""
        if not self.pending_acks:
            return
            
        # 按队列分组以优化批量操作
        from collections import defaultdict
        acks_by_queue = defaultdict(list)
        for queue, event_id in self.pending_acks:
            acks_by_queue[queue].append(event_id)
        
        self.pending_acks.clear()
        
        # 并发执行每个队列的批量ACK
        tasks = []
        
        for queue, event_ids in acks_by_queue.items():
            # 使用缓存的队列名称和高性能批量大小
            prefixed_queue = self._get_prefixed_queue_cached(queue)
            batch_size = 2000  # 高性能批量大小
            
            for i in range(0, len(event_ids), batch_size):
                batch = event_ids[i:i+batch_size]
                tasks.append(self.app.ep.async_redis_client.xack(prefixed_queue, prefixed_queue, *batch))
        
        if tasks:
            await asyncio.gather(*tasks, return_exceptions=True)
        
        self.last_flush_time = time.time()
    
    async def _flush_status_updates(self):
        """批量更新状态 - 优化使用MSET"""
        if not self.status_updates:
            return
            
        # 使用自适应批量大小
        batch_size = self.status_batch_size
        updates = self.status_updates[:batch_size]
        self.status_updates = self.status_updates[batch_size:]
        
        # 优化：使用MSET进行超高效批量更新
        mapping = {}
        expire_keys = []
        
        for event_id, status_dict in updates:
            key = f"{self._status_prefix}{event_id}"
            mapping[key] = ujson.dumps(status_dict)
            expire_keys.append(key)
        
        if mapping:
            # 使用pipeline批量设置值和过期时间
            pipeline = self.app.ep.async_redis_client.pipeline()
            pipeline.mset(mapping)
            
            # 批量设置过期时间
            for key in expire_keys:
                pipeline.expire(key, 3600)
            
            await pipeline.execute()
    
    async def _flush_data_updates(self):
        """批量更新数据 - 优化使用MSET"""
        if not self.data_updates:
            return
            
        # 使用自适应批量大小
        batch_size = self.data_batch_size
        updates = self.data_updates[:batch_size]
        self.data_updates = self.data_updates[batch_size:]
        
        # 优化：使用MSET
        mapping = {}
        expire_keys = []
        
        for event_id, data in updates:
            key = f"{self._result_prefix}{event_id}"
            mapping[key] = data
            expire_keys.append(key)
        
        if mapping:
            pipeline = self.app.ep.async_redis_client.pipeline()
            pipeline.mset(mapping)
            
            # 批量设置过期时间
            for key in expire_keys:
                pipeline.expire(key, 3600)
                
            await pipeline.execute()
    
    async def _flush_task_info_updates(self):
        """批量更新任务信息到Redis Hash - 新方法，合并STATUS和RESULT"""
        if not self.task_info_updates:
            return
            
        # 获取当前缓冲区的快照，避免并发修改问题
        updates_snapshot = dict(self.task_info_updates)
        self.task_info_updates.clear()
        
        # 使用pipeline批量处理
        pipeline = self.app.ep.async_redis_client.pipeline()
        
        # 处理每个任务的更新
        for event_id, updates in updates_snapshot.items():
            key = f"{self.prefix}:TASK:{event_id}"
            
            # 使用HSET更新多个字段
            if updates:
                pipeline.hset(key, mapping=updates)
                pipeline.expire(key, 3600)  # 1小时过期
        
        # 执行pipeline
        if pipeline:
            await pipeline.execute()
    
    async def _flush_all_buffers(self):
        """并发刷新所有缓冲区"""
        await asyncio.gather(
            self._flush_acks(),
            self._flush_status_updates(),
            self._flush_data_updates(),
            self._flush_task_info_updates(),  # 新增：刷新合并的任务信息
            return_exceptions=True
        )
    
    async def _collect_stats_async(self, queue: str, success: bool, processing_time: float, total_latency: float):
        """高性能异步统计收集，完全非阻塞"""
        try:
            if hasattr(self.app, 'consumer_manager') and self.app.consumer_manager:
                # 异步方式收集，不等待完成
                asyncio.create_task(self._update_stats_nonblocking(queue, success, processing_time, total_latency))
        except Exception:
            pass  # 统计错误不应影响主流程
    
    async def _update_stats_nonblocking(self, queue: str, success: bool, processing_time: float, total_latency: float):
        """非阻塞统计更新"""
        try:
            self.app.consumer_manager.task_finished(queue)
            self.app.consumer_manager.update_stats(
                queue=queue,
                success=success,
                processing_time=processing_time,
                total_latency=total_latency
            )
        except Exception as e:
            logger.debug(f"Stats collection error (non-critical): {e}")
    
        
    async def logic(self, semaphore: asyncio.Semaphore, event_id: str, event_data: dict, queue: str, routing: dict = None, consumer: str = None):
        """Process a single task"""
        try:
            # async with semaphore:
            task_name = event_data.get("name", "")
            task = self.app.get_task_by_name(task_name)
            if not task:
                exception = f"{task_name} {queue} {routing}未绑定任何task"
                logger.error(exception)
                await self._quick_ack(queue, event_id)
                
                # 任务不存在时也记录started_at（使用当前时间）
                current_time = time.time()
                trigger_time_float = float(event_data['trigger_time'])
                duration = current_time - trigger_time_float
                # 使用Hash更新
                self.task_info_updates[event_id] = {
                    "status": "error",
                    "exception": exception,
                    "started_at": str(current_time),
                    "completed_at": str(current_time),
                    "duration": str(duration),
                    "consumer": consumer,
                    "result": "null"  # 任务不存在时没有结果
                }
                await self._flush_task_info_updates()
                return
            
            self.pedding_count = await self.get_pending_count_cached(queue)
            
            status = "success"
            exception = None
            error_msg = None
            ret = None
            
            # 提前记录开始时间，避免在reject情况下未定义
            execution_start_time = time.time()
            
            args = ujson.loads(event_data["args"])
            kwargs = ujson.loads(event_data["kwargs"])
            
            # Execute lifecycle methods
            result = task.on_before(
                event_id=event_id,
                pedding_count=self.pedding_count,
                args=args,
                kwargs=kwargs,
            )
            if asyncio.iscoroutine(result):
                result = await result
                
            if result and result.reject:
                # 任务被reject，使用Hash更新
                self.task_info_updates[event_id] = {
                    "status": "rejected",
                    "consumer": consumer,
                    "started_at": str(execution_start_time),
                    "completed_at": str(time.time()),
                    "error_msg": "Task rejected by on_before"
                }
                await self._flush_task_info_updates()
                return
                
            # 标记任务开始执行
            if hasattr(self.app, 'consumer_manager') and self.app.consumer_manager:
                self.app.consumer_manager.task_started(queue)
            
            # 更新任务真正开始执行的时间（在on_before之后）
            execution_start_time = time.time()
            
            # 使用Hash更新running状态
            # 为了让用户能看到任务正在运行，立即写入running状态
            # running_key = f"{self.prefix}:TASK:{event_id}"
            # 保存开始信息，但不设置status为running，避免竞态条件
            self.task_info_updates[event_id] = {
                # "status": "running",  # 不在这里设置status
                "consumer": consumer,
                "started_at": str(execution_start_time)
            }
            # await self.app.ep.async_redis_client.hset(running_key, mapping={
            #     "status": "running",
            #     "consumer": consumer,
            #     "started_at": str(execution_start_time)
            # })
            
            try:
                
                task_result = task(event_id, event_data['trigger_time'], *args, **kwargs)
                if asyncio.iscoroutine(task_result):
                    ret = await task_result
                else:
                    ret = task_result
                result = task.on_success(
                    event_id=event_id,
                    args=args,
                    kwargs=kwargs,
                    result=ret,
                )
                if asyncio.iscoroutine(result):
                    await result
                    
            except SystemExit:
                # 处理系统退出信号，不记录为错误
                logger.info('Task interrupted by system exit')
                status = "interrupted"
                exception = "System exit"
                error_msg = "Task interrupted by shutdown"
            except Exception as e:
                logger.error('任务执行出错')
                status = "error"
                exception = traceback.format_exc()
                error_msg = str(e)
                traceback.print_exc()
                
            finally:
                # 添加到批量缓冲区而不是立即执行
                await self._quick_ack(queue, event_id)
                
                # 计算完成时间和消耗时间
                completed_at = time.time()
                trigger_time_float = float(event_data['trigger_time'])
                # 计算两个时间指标，确保不会出现负数
                execution_time = max(0, completed_at - execution_start_time)  # 实际执行时间
                total_latency = max(0, completed_at - trigger_time_float)     # 总延迟时间（包含等待）
                
                # 异步收集统计信息（高性能模式下非阻塞）
                await self._collect_stats_async(
                    queue=queue,
                    success=(status == "success"),
                    processing_time=execution_time,
                    total_latency=total_latency
                )
                
                # 使用Hash原子更新所有信息
                # 重要：先设置result，再设置status，确保不会出现status=success但result还没写入的情况
                task_info = {
                    "completed_at": str(completed_at),
                    "execution_time": str(execution_time),
                    "duration": str(total_latency),
                    "consumer": consumer,
                    'status': status
                }
                
                # 先写入结果
                if ret is None:
                    task_info["result"] = "null"  # JSON null
                else:
                    task_info["result"] = ret if isinstance(ret, str) else ujson.dumps(ret)
                
                # 再写入错误信息（如果有）
                if exception:
                    task_info["exception"] = exception
                if error_msg:
                    task_info["error_msg"] = error_msg
                    
                
                # 更新到缓冲区
                if event_id in self.task_info_updates:
                    # 合并更新（保留started_at等之前的信息）
                    # 重要：确保最终状态覆盖之前的running状态
                    self.task_info_updates[event_id].update(task_info)
                else:
                    self.task_info_updates[event_id] = task_info
                
                result = task.on_end(
                    event_id=event_id,
                    args=args,
                    kwargs=kwargs,
                    result=ret,
                    pedding_count=self.pedding_count,
                )
                if asyncio.iscoroutine(result):
                    await result
                    
                # Handle routing
                if routing:
                    agg_key = routing.get("agg_key")
                    routing_key = routing.get("routing_key")
                    if routing_key and agg_key:
                        # 避免在多进程环境下使用跨进程的锁
                        # 直接操作，依赖 Python GIL 和原子操作
                        if queue in self.app.ep.solo_running_state and routing_key in self.app.ep.solo_running_state[queue]:
                            self.app.ep.solo_running_state[queue][routing_key] -= 1
                    try:
                        if result and result.urgent_retry:
                            self.app.ep.solo_urgent_retry[routing_key] = True
                    except:
                        pass
                    if result and result.delay:
                        self.app.ep.task_scheduler[queue][routing_key] = time.time() + result.delay
                            
        finally:
            self.batch_counter -= 1
    
    async def loop(self):
        """Optimized main loop with dynamic batching"""
        semaphore = asyncio.Semaphore(self.concurrency)
        
        
        # Dynamic batch processing
        min_batch_size = 10   # 优化：降低最小批次
        max_batch_size = 500  # 优化：提高最大批次
        batch_size = 100
        tasks_batch = []
        
        # Performance tracking
        last_periodic_flush = time.time()
        last_batch_adjust = time.time()
        last_buffer_check = time.time()
        
        # 高性能缓冲区监控阈值
        max_buffer_size = 5000
        
        while True:
            # 检查是否需要退出
            if hasattr(self.app, '_should_exit') and self.app._should_exit:
                logger.info("AsyncioExecutor detected shutdown signal, exiting...")
                break
                
            # # 动态调整批处理大小
            current_time = time.time()
            if current_time - last_batch_adjust > 1.0:
                # 根据队列类型获取长度
                if isinstance(self.event_queue, deque):
                    queue_len = len(self.event_queue)
                elif isinstance(self.event_queue, asyncio.Queue):
                    queue_len = self.event_queue.qsize()
                else:
                    queue_len = 0
                    
                # 优化：更智能的动态调整
                if queue_len > 5000:
                    batch_size = min(max_batch_size, batch_size + 50)
                elif queue_len > 1000:
                    batch_size = min(max_batch_size, batch_size + 20)
                elif queue_len < 100:
                    batch_size = max(min_batch_size, batch_size - 20)
                elif queue_len < 500:
                    batch_size = max(min_batch_size, batch_size - 10)
                last_batch_adjust = current_time
                
            # 从队列获取事件
            event = None
            try:
                event = await asyncio.wait_for(self.event_queue.get(), timeout=0.1)
            except asyncio.TimeoutError:
                event = None
                    
            if event:
                event.pop("execute_time", None)
                tasks_batch.append(event)
            # 批量创建协程任务
            if tasks_batch:
                for event in tasks_batch:
                    self.batch_counter += 1
                    asyncio.create_task(self.logic(semaphore, **event))
                
                tasks_batch.clear()
            
            # 智能缓冲区管理和刷新
            buffer_full = (
                len(self.pending_acks) >= max_buffer_size or
                len(self.status_updates) >= max_buffer_size or 
                len(self.data_updates) >= max_buffer_size or
                len(self.task_info_updates) >= max_buffer_size  # 新增：检查Hash缓冲区
            )
            
            # 定期或缓冲区满时刷新
            should_flush_periodic = (current_time - last_periodic_flush > self.max_flush_interval)
            has_pending_data = (self.pending_acks or self.status_updates or self.data_updates or self.task_info_updates)
            
            if buffer_full or (should_flush_periodic and has_pending_data):
                asyncio.create_task(self._flush_all_buffers())
                last_periodic_flush = current_time
            
            
            # 智能休眠策略
            has_events = False
            if isinstance(self.event_queue, deque):
                has_events = bool(self.event_queue)
            elif isinstance(self.event_queue, asyncio.Queue):
                has_events = not self.event_queue.empty()
                
            if has_events:
                await asyncio.sleep(0)  # 有任务时立即切换
            else:
                # 检查是否需要立即刷新缓冲区
                if (self.pending_acks or self.status_updates or self.data_updates or self.task_info_updates):
                    await self._flush_all_buffers()
                await asyncio.sleep(0.001)  # 无任务时短暂休眠
        
        # 退出前清理所有缓冲区
        logger.info("AsyncioExecutor flushing buffers before exit...")
        await self._flush_all_buffers()
        
        # 等待所有正在执行的任务完成
        logger.info("AsyncioExecutor stopped")


import asyncio
import json
import logging
import os
import socket
import time
from typing import Dict, List, Optional, Any
from datetime import datetime, timezone
from collections import defaultdict

import redis.asyncio as redis
from redis.asyncio import Redis
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from sqlalchemy import text, select, update, insert, and_, or_

from jettask.webui.config import PostgreSQLConfig, RedisConfig
from jettask.webui.models import Base, Task

logger = logging.getLogger(__name__)


class PostgreSQLConsumer:
    """PostgreSQL消费者，从Redis队列消费任务并持久化到PostgreSQL"""
    
    def _get_next_sync_time(self, sync_check_count: int) -> float:
        """计算下次同步时间的时间戳（指数退避）"""
        # 指数退避：1, 2, 4, 8, 16, 32, 64 秒
        delay = min(2 ** sync_check_count, 64)
        return time.time() + delay
    
    def __init__(self, pg_config: PostgreSQLConfig, redis_config: RedisConfig, prefix: str = "jettask", node_id: str = None):
        self.pg_config = pg_config
        self.redis_config = redis_config
        self.prefix = prefix
        self.redis_client: Optional[Redis] = None
        self.async_engine = None
        self.AsyncSessionLocal = None
        self.consumer_group = f"{prefix}_pg_consumer"
        self.consumer_id = f"pg_consumer_{int(time.time()*1000)}"  # 使用时间戳确保唯一性
        # 节点标识，用于分布式环境
        import socket
        self.node_id = node_id or f"{socket.gethostname()}_{os.getpid()}"
        self._running = False
        self._tasks = []
        self._known_queues = set()  # 缓存已知的队列
        self._consecutive_errors = defaultdict(int)  # 记录每个队列的连续错误次数
        # 动态批次大小
        self.batch_size = 1000  # 初始批次大小
        self.min_batch_size = 100
        self.max_batch_size = 1000
        # 分区信息
        self.partition_info = {
            'min_id': None,
            'max_id': None,
            'id_ranges': [],  # [(start, end), ...]
            'active_nodes': 1,
            'node_index': 0,
            'last_update': 0,
            'is_last_node': False,  # 是否是最后一个节点
            'grace_period': True,  # 宽限期标志
            'grace_start': 0  # 宽限期开始时间
        }
        self.pubsub = None  # Pub/Sub客户端
        
    async def start(self):
        """启动消费者"""
        logger.info(f"Starting PostgreSQL consumer on node: {self.node_id}")
        
        # 使用 Redis 协调后不需要启动延迟
        
        # 连接Redis
        self.redis_client = await redis.Redis(
            host=self.redis_config.host,
            port=self.redis_config.port,
            db=self.redis_config.db,
            password=self.redis_config.password,
            decode_responses=False
        )
        
        # 创建SQLAlchemy异步引擎 - 优化配置
        # 将 DSN 转换为 SQLAlchemy 格式
        if self.pg_config.dsn.startswith('postgresql://'):
            dsn = self.pg_config.dsn.replace('postgresql://', 'postgresql+psycopg://', 1)
        else:
            dsn = self.pg_config.dsn
            
        self.async_engine = create_async_engine(
            dsn,
            pool_size=20,      # 连接池大小
            max_overflow=10,   # 最大溢出连接数
            pool_pre_ping=True,  # 连接前检查
            pool_recycle=300,    # 5分钟回收连接
            echo=False
        )
        
        # 创建异步会话工厂
        self.AsyncSessionLocal = sessionmaker(
            self.async_engine,
            class_=AsyncSession,
            expire_on_commit=False
        )
        
        # 初始化数据库架构
        await self._init_database()
        
        self._running = True
        
        # 创建Pub/Sub客户端用于节点通知
        self.pubsub = self.redis_client.pubsub()
        await self.pubsub.subscribe(f"{self.prefix}:NODE_EVENTS")
        
        # 启动消费任务
        self._tasks = [
            asyncio.create_task(self._consume_queues()),
            asyncio.create_task(self._update_task_status()),
            asyncio.create_task(self._partition_coordinator()),  # 分区协调器
            asyncio.create_task(self._database_maintenance()),  # 数据库维护任务
            asyncio.create_task(self._node_event_listener())    # 节点事件监听器
        ]
        
        logger.info("PostgreSQL consumer started successfully")
        
    async def stop(self):
        """停止消费者"""
        logger.info("Stopping PostgreSQL consumer...")
        self._running = False
        
        # 发布节点离开事件
        try:
            await self.redis_client.publish(
                f"{self.prefix}:NODE_EVENTS", 
                json.dumps({"event": "node_leave", "node_id": self.node_id})
            )
            # 删除心跳键
            await self.redis_client.delete(f"{self.prefix}:NODE:{self.node_id}")
            logger.info(f"Published node leave event for {self.node_id}")
        except Exception as e:
            logger.error(f"Error publishing node leave event: {e}")
        
        # 取消所有任务
        for task in self._tasks:
            task.cancel()
        
        # 等待任务完成
        await asyncio.gather(*self._tasks, return_exceptions=True)
        
        # 关闭连接
        if self.pubsub:
            await self.pubsub.unsubscribe()
            await self.pubsub.close()
            
        if self.redis_client:
            await self.redis_client.close()
        
        if self.async_engine:
            await self.async_engine.dispose()
            
        logger.info("PostgreSQL consumer stopped")
        
    async def _init_database(self):
        """初始化数据库架构"""
        schema_path = "/home/yuyang/easy-task/jettask/webui/schema.sql"
        try:
            with open(schema_path, 'r') as f:
                schema_sql = f.read()
            
            async with self.AsyncSessionLocal() as session:
                await session.execute(text(schema_sql))
                await session.commit()
                logger.info("Database schema initialized")
        except FileNotFoundError:
            logger.warning(f"Schema file not found at {schema_path}, skipping initialization")
        except Exception as e:
            logger.error(f"Failed to initialize database schema: {e}")
            
    async def _discover_queues(self):
        """定期发现新队列"""
        while self._running:
            try:
                pattern = f"{self.prefix}:QUEUE:*"
                new_queues = set()
                
                async for key in self.redis_client.scan_iter(match=pattern, count=100):
                    queue_name = key.decode('utf-8').split(":")[-1]
                    new_queues.add(queue_name)
                
                # 为新发现的队列创建消费者组
                for queue in new_queues - self._known_queues:
                    stream_key = f"{self.prefix}:QUEUE:{queue}"
                    try:
                        await self.redis_client.xgroup_create(
                            stream_key, self.consumer_group, id='0', mkstream=True
                        )
                        logger.info(f"Created consumer group for new queue: {queue}")
                    except redis.ResponseError:
                        # 消费者组已存在
                        pass
                
                self._known_queues = new_queues
                
                # 每30秒发现一次新队列
                await asyncio.sleep(30)
                
            except Exception as e:
                logger.error(f"Error discovering queues: {e}")
                await asyncio.sleep(10)
    
    async def _consume_queue(self, queue_name: str):
        """消费单个队列的任务"""
        stream_key = f"{self.prefix}:QUEUE:{queue_name}"
        check_backlog = True  # 是否检查历史消息
        lastid = "0-0"  # 初始ID
        
        while self._running and queue_name in self._known_queues:
            try:
                # 决定从哪里开始读取
                if check_backlog:
                    myid = lastid
                else:
                    myid = ">"
                
                # 读取消息
                messages = await self.redis_client.xreadgroup(
                    self.consumer_group,
                    self.consumer_id,
                    {stream_key: myid},
                    count=1000,  # 每次最多读取50条
                    block=1000 if not check_backlog else 0  # 有历史消息时不阻塞，否则阻塞1秒
                )
                
                # 检查是否还有更多历史消息
                if not messages or (messages and len(messages[0][1]) == 0):
                    check_backlog = False
                    continue
                
                # 处理消息
                if messages:
                    await self._process_messages(messages)
                    # 重置错误计数
                    self._consecutive_errors[queue_name] = 0
                    
                    # 更新lastid为最后一条消息的ID
                    if messages[0] and messages[0][1]:
                        lastid = messages[0][1][-1][0].decode('utf-8') if isinstance(messages[0][1][-1][0], bytes) else messages[0][1][-1][0]
                        
                        # 如果读取到了完整批次，说明可能还有更多历史消息
                        if len(messages[0][1]) >= 50:
                            check_backlog = True
                        else:
                            check_backlog = False
                    
            except redis.ResponseError as e:
                if "NOGROUP" in str(e):
                    # 消费者组不存在，尝试创建
                    try:
                        await self.redis_client.xgroup_create(
                            stream_key, self.consumer_group, id='0', mkstream=True
                        )
                        logger.info(f"Recreated consumer group for queue: {queue_name}")
                        # 重置状态，从头开始读取
                        check_backlog = True
                        lastid = "0-0"
                    except:
                        pass
                else:
                    logger.error(f"Redis error for queue {queue_name}: {e}")
                    self._consecutive_errors[queue_name] += 1
                    
                # 如果连续错误太多，暂时跳过这个队列
                if self._consecutive_errors[queue_name] > 10:
                    logger.warning(f"Too many errors for queue {queue_name}, will retry later")
                    await asyncio.sleep(30)
                    self._consecutive_errors[queue_name] = 0
                # else:
                #     await asyncio.sleep(1)
                    
            except Exception as e:
                logger.error(f"Error consuming queue {queue_name}: {e}", exc_info=True)
                self._consecutive_errors[queue_name] += 1
                await asyncio.sleep(1)
    
    async def _consume_queues(self):
        """启动所有队列的消费任务"""
        # 启动队列发现任务
        discover_task = asyncio.create_task(self._discover_queues())
        
        # 为每个队列创建独立的消费任务
        queue_tasks = {}
        
        while self._running:
            try:
                # 为新队列创建任务
                for queue in self._known_queues:
                    if queue not in queue_tasks or queue_tasks[queue].done():
                        queue_tasks[queue] = asyncio.create_task(self._consume_queue(queue))
                        logger.info(f"Started consumer task for queue: {queue}")
                
                # 清理已完成或已移除的队列任务
                for queue in list(queue_tasks.keys()):
                    if queue not in self._known_queues:
                        queue_tasks[queue].cancel()
                        del queue_tasks[queue]
                        logger.info(f"Stopped consumer task for removed queue: {queue}")
                
                # 等待一段时间再检查
                await asyncio.sleep(10)
                
            except Exception as e:
                logger.error(f"Error in consume_queues manager: {e}")
                await asyncio.sleep(5)
        
        # 清理所有任务
        discover_task.cancel()
        for task in queue_tasks.values():
            task.cancel()
        
        await asyncio.gather(discover_task, *queue_tasks.values(), return_exceptions=True)
                
    async def _process_messages(self, messages: List):
        """处理消息并保存到PostgreSQL"""
        tasks_to_insert = []
        
        for stream_key, stream_messages in messages:
            if not stream_messages:
                logger.debug(f"No messages in stream {stream_key}")
                continue
                
            # 处理stream_key
            if isinstance(stream_key, bytes):
                stream_key_str = stream_key.decode('utf-8')
            else:
                stream_key_str = stream_key
                
            queue_name = stream_key_str.split(":")[-1]
            
            for msg_id, data in stream_messages:
                try:
                    # 跳过空消息
                    if not msg_id or not data:
                        logger.warning(f"Skipping empty message in queue {queue_name}")
                        continue
                        
                    # 解析任务数据
                    task_data = {
                        k.decode('utf-8') if isinstance(k, bytes) else k: 
                        v.decode('utf-8') if isinstance(v, bytes) else v
                        for k, v in data.items()
                    }
                    
                    # 处理消息ID
                    msg_id_str = msg_id.decode('utf-8') if isinstance(msg_id, bytes) else str(msg_id)
                    
                    # 获取task_name，优先使用name字段
                    task_name = task_data.get('name', task_data.get('task', 'unknown'))
                    logger.debug(f"Task {msg_id_str} name: {task_name}, available fields: {list(task_data.keys())}")
                    
                    # 提取trigger_time时间戳并转换为UTC时间作为created_at
                    created_at = None
                    if 'trigger_time' in task_data:
                        try:
                            timestamp = float(task_data['trigger_time'])
                            # 将时间戳转换为UTC datetime对象
                            created_at = datetime.fromtimestamp(timestamp, tz=timezone.utc)
                        except (ValueError, TypeError):
                            logger.warning(f"Invalid trigger_time value: {task_data['trigger_time']}")
                    
                    # 准备插入数据
                    task_info = {
                        'id': msg_id_str,
                        'queue_name': queue_name,
                        'task_name': task_name,
                        'task_data': json.dumps(task_data),
                        'priority': int(task_data.get('priority', 0)),
                        'retry_count': int(task_data.get('retry', 0)),
                        'max_retry': int(task_data.get('max_retry', 3)),
                        'status': 'pending',
                        'metadata': json.dumps(task_data.get('metadata', {})),
                        'created_at': created_at  # UTC datetime对象
                    }
                    logger.info(f'准备插入数据: {created_at=}')
                    tasks_to_insert.append(task_info)
                    
                    # 确认消息
                    await self.redis_client.xack(stream_key, self.consumer_group, msg_id)
                    
                except Exception as e:
                    logger.error(f"Error processing message {msg_id}: {e}")
                    
        # 批量插入到PostgreSQL
        if tasks_to_insert:
            await self._insert_tasks(tasks_to_insert)
            
    async def _insert_tasks(self, tasks: List[Dict[str, Any]]):
        """批量插入任务到PostgreSQL"""
        if not tasks:
            return
            
        try:
            async with self.AsyncSessionLocal() as session:
                # 使用原生SQL执行批量插入（SQLAlchemy的批量插入对于UPSERT比较复杂）
                query = text("""
                    INSERT INTO tasks (id, queue_name, task_name, task_data, priority, 
                                     retry_count, max_retry, status, metadata, created_at)
                    VALUES (:id, :queue_name, :task_name, CAST(:task_data AS jsonb), :priority, 
                           :retry_count, :max_retry, :status, CAST(:metadata AS jsonb), :created_at)
                    ON CONFLICT (id) DO UPDATE SET
                        task_name = EXCLUDED.task_name,
                        retry_count = EXCLUDED.retry_count,
                        metadata = EXCLUDED.metadata
                """)
                
                # 准备数据
                for task in tasks:
                    await session.execute(query, {
                        'id': task['id'],
                        'queue_name': task['queue_name'],
                        'task_name': task['task_name'],
                        'task_data': task['task_data'],  # 已经是JSON字符串
                        'priority': task['priority'],
                        'retry_count': task['retry_count'],
                        'max_retry': task['max_retry'],
                        'status': task['status'],
                        'metadata': task['metadata'],  # 已经是JSON字符串
                        'created_at': task.get('created_at')
                    })
                
                await session.commit()
                logger.info(f"Inserted {len(tasks)} tasks to PostgreSQL")
                
                # 调试：检查第一个任务的数据
                if tasks:
                    logger.debug(f"First task data: id={tasks[0]['id']}, name={tasks[0]['task_name']}")
                
        except Exception as e:
            logger.error(f"Error inserting tasks to PostgreSQL: {e}")
            
    async def _update_task_status(self):
        """定期更新任务状态"""
        # 查询结果缓存
        query_cache = {
            'tasks': [],
            'last_update': 0,
            'cache_ttl': 1.0  # 1秒缓存
        }
        
        while self._running:
            try:
                
                
                # 1. 等待分区信息就绪
                if self.partition_info['active_nodes'] == 0:
                    logger.info("Waiting for partition info...")
                    await asyncio.sleep(2)
                    continue
                
                # 2. 宽限期处理 - 重新上线的节点暂不处理任务
                if self.partition_info.get('grace_period', False):
                    grace_elapsed = time.time() - self.partition_info.get('grace_start', 0)
                    if grace_elapsed < 10:  # 10秒宽限期
                        logger.info(f"Node in grace period, waiting... ({10 - grace_elapsed:.1f}s remaining)")
                        await asyncio.sleep(1)
                        continue
                    else:
                        # 宽限期结束
                        self.partition_info['grace_period'] = False
                        logger.info("Grace period ended, resuming normal operation")
                
                # 2. 从PostgreSQL查询分配给当前节点的任务
                start_time = time.time()
                
                # 检查缓存是否有效
                if (query_cache['tasks'] and 
                    time.time() - query_cache['last_update'] < query_cache['cache_ttl']):
                    # 使用缓存的任务
                    unfinished_tasks = query_cache['tasks']
                    logger.debug(f"Using cached tasks: {len(unfinished_tasks)} tasks")
                else:
                    async with self.AsyncSessionLocal() as session:
                        # 设置语句超时
                        await session.execute(text("SET statement_timeout = '5000'"))  # 5秒超时
                        
                        # 使用更简单的分区策略：基于ID字符比较
                        # 将ID空间平均分成N份
                        if self.partition_info['is_last_node']:
                            # 最后一个节点：处理自己的分区 + 所有新任务
                            # 基于ID中'-'后面的数字进行分区
                            query = text("""
                                SELECT id, sync_check_count 
                                FROM tasks 
                                WHERE status IN ('pending', 'running')
                                  AND next_sync_time <= :current_time
                                  AND (
                                    -- 属于当前节点的分区（基于-后面的数字取模）
                                    MOD(CAST(SPLIT_PART(id, '-', 2) AS INTEGER), :active_nodes) = :node_index
                                    -- 或者是新任务（ID大于max_id）
                                    OR id > :max_id
                                  )
                                ORDER BY next_sync_time
                                LIMIT :batch_size
                                """)
                            result = await session.execute(query, {
                                'batch_size': self.batch_size,
                                'current_time': time.time(),
                                'active_nodes': self.partition_info['active_nodes'],
                                'node_index': self.partition_info['node_index'],
                                'max_id': self.partition_info['max_id'] or ''
                            })
                            unfinished_tasks = result.mappings().all()
                        else:
                            # 其他节点：只处理自己的分区
                            # 基于ID中'-'后面的数字进行分区
                            query = text("""
                                SELECT id, sync_check_count 
                                FROM tasks 
                                WHERE status IN ('pending', 'running')
                                  AND next_sync_time <= :current_time
                                  AND MOD(CAST(SPLIT_PART(id, '-', 2) AS INTEGER), :active_nodes) = :node_index
                                ORDER BY next_sync_time
                                LIMIT :batch_size
                                """)
                            result = await session.execute(query, {
                                'batch_size': self.batch_size,
                                'current_time': time.time(),
                                'active_nodes': self.partition_info['active_nodes'],
                                'node_index': self.partition_info['node_index']
                            })
                            unfinished_tasks = result.mappings().all()
                        
                        # 更新缓存
                        query_cache['tasks'] = unfinished_tasks
                        query_cache['last_update'] = time.time()
                    
                    
                if not unfinished_tasks:
                    continue
                    
                query_time = time.time() - start_time
                logger.info(f"Node {self.partition_info['node_index']}/{self.partition_info['active_nodes']}: "
                          f"Found {len(unfinished_tasks)} tasks "
                          f"{'(including new tasks)' if self.partition_info['is_last_node'] else ''} "
                          f"查询耗时: {query_time:.2f}秒")
                
                # 性能警告
                if query_time > 1.0:
                    logger.warning(f"Database query took {query_time:.2f}s, which is slow. "
                                 f"Consider running ANALYZE on the tasks table.")

                # 3. 批量获取任务的Redis数据（不需要加锁，因为使用了分区）
                start_time = time.time()
                pipeline = self.redis_client.pipeline()
                task_infos = []
                
                for task in unfinished_tasks:
                    # Now using mappings() so we can access by name
                    task_id = task['id']
                    sync_check_count = task['sync_check_count'] or 0
                    task_infos.append({
                        'id': task_id,
                        'sync_check_count': sync_check_count
                    })
                    # 从Hash获取所有字段
                    task_key = f"{self.prefix}:TASK:{task_id}"
                    pipeline.hgetall(task_key)
                    
                # 执行批量查询
                redis_values = await pipeline.execute()
                logger.info(f'遍历redis耗时： {time.time() - start_time:.2f}秒')
                
                # 4. 处理查询结果
                updates = []
                no_change_tasks = []  # 没有变化的任务，需要更新next_sync_time
                
                for i, task_info in enumerate(task_infos):
                    task_id = task_info['id']
                    sync_check_count = task_info['sync_check_count']
                    hash_data = redis_values[i]  # TASK Hash数据
                    
                    update_info = {
                        'id': task_id,
                        'status': None,
                        'result': None,
                        'error_message': None,
                        'started_at': None,
                        'completed_at': None,
                        'worker_id': None,
                        'execution_time': None,
                        'duration': None,
                        'metadata': None
                    }
                    
                    # 优先处理新的Hash数据
                    if hash_data:
                        try:
                            # Hash数据已经是字典格式，直接使用
                            # 解码bytes键和值
                            hash_dict = {}
                            for k, v in hash_data.items():
                                key = k.decode('utf-8') if isinstance(k, bytes) else k
                                value = v.decode('utf-8') if isinstance(v, bytes) else v
                                hash_dict[key] = value
                            
                            # 映射字段
                            update_info['status'] = hash_dict.get('status')
                            update_info['error_message'] = hash_dict.get('error_msg') or hash_dict.get('exception')
                            
                            # 转换时间戳字符串为UTC datetime
                            if hash_dict.get('started_at'):
                                update_info['started_at'] = datetime.fromtimestamp(float(hash_dict['started_at']), tz=timezone.utc)
                            if hash_dict.get('completed_at'):
                                update_info['completed_at'] = datetime.fromtimestamp(float(hash_dict['completed_at']), tz=timezone.utc)
                                
                            update_info['worker_id'] = hash_dict.get('consumer') or hash_dict.get('worker_id')
                            
                            # 转换数值字符串为浮点数
                            if hash_dict.get('execution_time'):
                                update_info['execution_time'] = float(hash_dict['execution_time'])
                            if hash_dict.get('duration'):
                                update_info['duration'] = float(hash_dict['duration'])
                            
                            # 处理result字段
                            if 'result' in hash_dict:
                                result_str = hash_dict['result']
                                if result_str == 'null':
                                    update_info['result'] = None
                                else:
                                    update_info['result'] = result_str
                            
                            logger.debug(f"Task {task_id} from hash: {hash_dict}")
                            
                        except Exception as e:
                            logger.warning(f"Failed to parse hash data for task {task_id}: {e}")
                    
                    
                    # 只有当有数据更新时才添加到更新列表（任何字段有值都应该更新）
                    if any(v is not None for k, v in update_info.items() if k not in ['id', 'sync_check_count']):
                        update_info['sync_check_count'] = sync_check_count
                        updates.append(update_info)
                    else:
                        # 没有变化的任务
                        no_change_tasks.append({
                            'id': task_id,
                            'sync_check_count': sync_check_count
                        })
                
                # 5. 批量更新任务状态
                if updates:
                    start_time = time.time()
                    await self._update_tasks(updates)
                    elapsed_time = time.time() - start_time
                    logger.info(f"Updated {len(updates)} task statuses from {len(task_infos)} processed 耗时={elapsed_time:.2f}秒")
                    
                    # 动态调整批次大小
                    if elapsed_time > 5:  # 处理时间超过5秒，减小批次
                        self.batch_size = max(self.min_batch_size, int(self.batch_size * 0.8))
                        logger.info(f"Decreased batch size to {self.batch_size} due to slow processing")
                    elif elapsed_time < 1 and len(unfinished_tasks) == self.batch_size:  # 处理很快且达到批次上限
                        self.batch_size = min(self.max_batch_size, int(self.batch_size * 1.2))
                        logger.info(f"Increased batch size to {self.batch_size} due to fast processing")
                
                # 5. 更新没有变化的任务的下次同步时间
                if no_change_tasks:
                    await self._update_next_sync_time(no_change_tasks)
                    logger.debug(f"Updated next_sync_time for {len(no_change_tasks)} unchanged tasks")
                
                # 6. 动态等待时间
                if not updates and not no_change_tasks:
                    # 没有需要处理的任务，等待一段时间
                    await asyncio.sleep(2)
                else:
                    # 有任务处理，继续处理下一批
                    await asyncio.sleep(0.01)
                    
            except Exception as e:
                logger.error(f"Error in update_task_status: {e}", exc_info=True)
                
    async def _update_tasks(self, updates: List[Dict[str, Any]]):
        """批量更新任务状态"""
        if not updates:
            return
            
        try:
            async with self.AsyncSessionLocal() as session:
                # 根据状态分组更新，简化SQL
                completed_tasks = []
                running_tasks = []
                
                for update in updates:
                    task_id = update['id']
                    status = update.get('status')
                    
                    
                    if status in ('success', 'error', 'failed', 'timeout', 'cancelled'):
                        completed_tasks.append(update)
                    else:
                        running_tasks.append(update)
                
                # 批量更新已完成的任务
                if completed_tasks:
                    # 使用循环执行更新（SQLAlchemy的批量更新对于复杂的UPDATE FROM VALUES比较困难）
                    for task in completed_tasks:
                        query = text("""
                            UPDATE tasks SET
                                status = :status,
                                result = CAST(:result AS jsonb),
                                error_message = :error_message,
                                started_at = COALESCE(:started_at, started_at),
                                completed_at = :completed_at,
                                worker_id = COALESCE(:worker_id, worker_id),
                                execution_time = :execution_time,
                                duration = :duration,
                                sync_check_count = 0,
                                next_sync_time = 999999999999
                            WHERE id = :id
                        """)
                        
                        await session.execute(query, {
                            'id': task['id'],
                            'status': task.get('status'),
                            'result': task.get('result'),  # 已经是JSON字符串
                            'error_message': task.get('error_message'),
                            'started_at': task.get('started_at'),
                            'completed_at': task.get('completed_at'),
                            'worker_id': task.get('worker_id'),
                            'execution_time': task.get('execution_time'),
                            'duration': task.get('duration')
                        })
                    
                    await session.commit()
                    logger.debug(f"Updated {len(completed_tasks)} completed tasks")
                
                # 批量更新运行中的任务
                if running_tasks:
                    # 只更新有变化的字段
                    task_ids = [t['id'] for t in running_tasks]
                    # 准备更新数据
                    for task in running_tasks:
                        sync_count = task.get('sync_check_count', 0) + 1
                        next_sync = self._get_next_sync_time(sync_count)
                        
                        query = text("""
                            UPDATE tasks
                            SET sync_check_count = :sync_check_count,
                                next_sync_time = :next_sync_time
                            WHERE id = :id
                        """)
                        
                        await session.execute(query, {
                            'id': task['id'],
                            'sync_check_count': sync_count,
                            'next_sync_time': next_sync
                        })
                    
                    await session.commit()
                    logger.debug(f"Updated {len(running_tasks)} running tasks")
                
                logger.info(f"Updated {len(updates)} task statuses")
                
        except Exception as e:
            logger.error(f"Error updating task statuses: {e}")

    async def _update_next_sync_time(self, tasks: List[Dict[str, Any]]):
        """更新没有变化的任务的下次同步时间"""
        if not tasks:
            return
            
        # 准备更新数据
        update_data = []
        for task in tasks:
            sync_count = task.get('sync_check_count', 0) + 1
            next_sync = self._get_next_sync_time(sync_count)
            update_data.append((task['id'], sync_count, next_sync))
        
        try:
            async with self.AsyncSessionLocal() as session:
                # 使用循环执行更新
                for task_id, sync_count, next_sync in update_data:
                    query = text("""
                        UPDATE tasks
                        SET sync_check_count = :sync_check_count,
                            next_sync_time = :next_sync_time
                        WHERE id = :id
                    """)
                    
                    await session.execute(query, {
                        'id': task_id,
                        'sync_check_count': sync_count,
                        'next_sync_time': next_sync
                    })
                
                await session.commit()
                logger.debug(f"Updated next_sync_time for {len(tasks)} unchanged tasks")
        except Exception as e:
            logger.error(f"Error updating next_sync_time: {e}")

    async def _partition_coordinator(self):
        """分区协调器 - 定期统计任务并分配分区
        
        分区策略：
        1. 使用 MD5(id) 的哈希值对节点数取模，确保任务均匀分布
        2. 最后一个节点额外处理所有新增的任务（ID > max_id）
        3. 这样既保证了负载均衡，又确保新任务能被及时处理
        
        PostgreSQL中的哈希计算：
        MOD(('x' || substr(md5(id), 1, 8))::bit(32)::int, node_count) = node_index
        - md5(id): 计算ID的MD5哈希
        - substr(..., 1, 8): 取前8个字符（32位）
        - 'x' || ...: 添加前缀避免负数
        - ::bit(32)::int: 转换为整数
        - MOD(..., node_count): 对节点数取模
        """
        partition_key = f"{self.prefix}:PARTITION_INFO"
        node_heartbeat_key = f"{self.prefix}:NODE:{self.node_id}"
        
        while self._running:
            try:
                # 1. 更新节点心跳
                is_new_node = not await self.redis_client.exists(node_heartbeat_key)
                await self.redis_client.setex(node_heartbeat_key, 30, time.time())
                
                # 如果是新节点或从离线恢复
                if is_new_node:
                    # 检查是否是重新上线（通过检查是否有历史分区信息）
                    stored_info = await self.redis_client.hget(partition_key, self.node_id)
                    if stored_info:
                        # 是重新上线的节点，设置宽限期
                        logger.warning(f"Node {self.node_id} is rejoining after offline, entering grace period")
                        self.partition_info['grace_period'] = True
                        self.partition_info['grace_start'] = time.time()
                    
                    await self.redis_client.publish(
                        f"{self.prefix}:NODE_EVENTS", 
                        json.dumps({"event": "node_join", "node_id": self.node_id})
                    )
                    logger.info(f"Published node join event for {self.node_id}")
                
                # 2. 获取所有活跃节点
                pattern = f"{self.prefix}:NODE:*"
                active_nodes = []
                async for key in self.redis_client.scan_iter(match=pattern):
                    heartbeat = await self.redis_client.get(key)
                    if heartbeat:
                        node_id = key.decode('utf-8').split(':')[-1]
                        active_nodes.append(node_id)
                
                active_nodes.sort()  # 确保所有节点看到相同的顺序
                node_count = len(active_nodes)
                
                if self.node_id in active_nodes:
                    node_index = active_nodes.index(self.node_id)
                else:
                    logger.error(f"Node {self.node_id} not in active nodes list")
                    await asyncio.sleep(5)
                    continue
                
                # 3. 检测节点数量变化，快速响应
                if node_count != self.partition_info['active_nodes']:
                    logger.info(f"Node count changed from {self.partition_info['active_nodes']} to {node_count}, updating partition immediately")
                    self.partition_info['last_update'] = 0  # 强制立即更新
                
                # 4. 获取任务ID范围（每30秒更新一次，或节点数变化时立即更新）
                current_time = time.time()
                if current_time - self.partition_info['last_update'] > 30:
                    async with self.AsyncSessionLocal() as session:
                        # 获取当前任务的ID范围
                        query = text("""
                            SELECT MIN(id) as min_id, MAX(id) as max_id
                            FROM tasks 
                            WHERE status IN ('pending', 'running')
                              AND next_sync_time <= :current_time
                        """)
                        result_obj = await session.execute(query, {'current_time': time.time()})
                        result = result_obj.mappings().fetchone()
                        
                        if result and result['min_id'] and result['max_id']:
                            min_id = result['min_id']
                            max_id = result['max_id']
                            
                            # 4. 基于ID计算分区
                            # 将ID空间分成N份
                            id_ranges = []
                            
                            # 对于字符串ID，使用哈希分区
                            for i in range(node_count):
                                is_last = (i == node_count - 1)
                                id_ranges.append({
                                    'node_index': i,
                                    'is_last': is_last
                                })
                            
                            # 当前节点的信息
                            current_node = id_ranges[node_index]
                            
                            self.partition_info.update({
                                'min_id': min_id,
                                'max_id': max_id,
                                'active_nodes': node_count,
                                'node_index': node_index,
                                'is_last_node': current_node['is_last'],
                                'last_update': current_time
                            })
                            
                            # 5. 发布分区信息到 Redis（供监控使用）
                            await self.redis_client.hset(partition_key, self.node_id, json.dumps({
                                'node_index': node_index,
                                'total_nodes': node_count,
                                'is_last_node': current_node['is_last'],
                                'updated_at': current_time
                            }))
                            
                            logger.info(f"Partition updated: node {node_index}/{node_count}, "
                                      f"is_last_node: {current_node['is_last']}")
                
                # 每5秒检查一次
                await asyncio.sleep(5)
                
            except Exception as e:
                logger.error(f"Error in partition coordinator: {e}", exc_info=True)
                await asyncio.sleep(10)
    
    async def _database_maintenance(self):
        """定期执行数据库维护任务"""
        last_analyze_time = 0
        analyze_interval = 3600  # 每小时执行一次ANALYZE
        
        while self._running:
            try:
                current_time = time.time()
                
                # 定期执行ANALYZE来更新统计信息
                if current_time - last_analyze_time > analyze_interval:
                    async with self.AsyncSessionLocal() as session:
                        logger.info("Running ANALYZE on tasks table to update statistics...")
                        await session.execute(text("ANALYZE tasks"))
                        await session.commit()
                        logger.info("ANALYZE completed successfully")
                        last_analyze_time = current_time
                
                # 每5分钟检查一次
                await asyncio.sleep(300)
                
            except Exception as e:
                logger.error(f"Error in database maintenance: {e}")
                await asyncio.sleep(60)
    
    async def _node_event_listener(self):
        """监听节点事件，实时响应节点变化"""
        while self._running:
            try:
                # 监听Pub/Sub消息
                message = await self.pubsub.get_message(ignore_subscribe_messages=True, timeout=1.0)
                
                if message and message['type'] == 'message':
                    try:
                        data = json.loads(message['data'])
                        event_type = data.get('event')
                        node_id = data.get('node_id')
                        
                        if event_type in ['node_join', 'node_leave']:
                            logger.info(f"Received {event_type} event from node {node_id}")
                            # 强制立即更新分区信息
                            self.partition_info['last_update'] = 0
                            
                            # 如果正在处理任务，等待当前批次完成
                            await asyncio.sleep(0.5)
                            
                    except Exception as e:
                        logger.error(f"Error processing node event: {e}")
                        
            except asyncio.TimeoutError:
                # 没有消息，继续循环
                pass
            except Exception as e:
                logger.error(f"Error in node event listener: {e}")
                await asyncio.sleep(1)


async def run_pg_consumer(pg_config: PostgreSQLConfig, redis_config: RedisConfig):
    """运行PostgreSQL消费者"""
    consumer = PostgreSQLConsumer(pg_config, redis_config)
    
    try:
        await consumer.start()
        
        # 保持运行
        while True:
            await asyncio.sleep(1)
            
    except KeyboardInterrupt:
        logger.info("Received interrupt signal")
    finally:
        await consumer.stop()

def main():
    """主入口函数"""
    import os
    from dotenv import load_dotenv
    
    load_dotenv()
    
    # 配置日志
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    
    # 创建配置
    pg_config = PostgreSQLConfig(
        host=os.getenv('JETTASK_PG_HOST', 'localhost'),
        port=int(os.getenv('JETTASK_PG_PORT', '5432')),
        database=os.getenv('JETTASK_PG_DB', 'jettask'),
        user=os.getenv('JETTASK_PG_USER', 'jettask'),
        password=os.getenv('JETTASK_PG_PASSWORD', '123456'),
    )
    
    redis_config = RedisConfig(
        host=os.getenv('REDIS_HOST', 'localhost'),
        port=int(os.getenv('REDIS_PORT', '6379')),
        db=int(os.getenv('REDIS_DB', '0')),
        password=os.getenv('REDIS_PASSWORD'),
    )
    
    # 运行消费者
    asyncio.run(run_pg_consumer(pg_config, redis_config))

if __name__ == '__main__':
    main()


#!/usr/bin/env python
"""
JetTask CLI - 命令行接口
"""
import click
import sys
import os
import importlib
import importlib.util
import json
from pathlib import Path

@click.group()
@click.version_option(version='0.1.0', prog_name='JetTask')
def cli():
    """JetTask - 高性能分布式任务队列系统"""
    pass

@cli.command()
@click.option('--host', default='0.0.0.0', help='服务器监听地址')
@click.option('--port', default=8001, type=int, help='服务器监听端口')
def webui(host, port):
    """启动 Web UI 监控界面"""
    from jettask.webui.backend.main import run_server
    click.echo(f"Starting JetTask Web UI on {host}:{port}")
    
    # 修改端口设置
    import uvicorn
    uvicorn.run(
        "jettask.webui.backend.main:app",
        host=host,
        port=port,
        log_level="info"
    )

def load_module_from_path(module_path: str):
    """从文件路径加载 Python 模块"""
    path = Path(module_path).resolve()
    
    if not path.exists():
        raise FileNotFoundError(f"Module file not found: {module_path}")
    
    # 获取模块名
    module_name = path.stem
    
    # 加载模块
    spec = importlib.util.spec_from_file_location(module_name, path)
    if spec is None:
        raise ImportError(f"Cannot load module from {module_path}")
    
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    
    return module

def find_jettask_app(module):
    """在模块中查找 Jettask 实例"""
    from jettask import Jettask
    
    # 查找模块中的 Jettask 实例
    for name in dir(module):
        obj = getattr(module, name)
        if isinstance(obj, Jettask):
            return obj
    
    # 如果没有找到，尝试查找名为 'app' 的变量
    if hasattr(module, 'app'):
        obj = getattr(module, 'app')
        if isinstance(obj, Jettask):
            return obj
    
    return None

@cli.command()
@click.argument('app_str')
@click.option('--queues', '-q', required=True, help='队列名称（逗号分隔，如: queue1,queue2）')
@click.option('--executor', '-e', 
              type=click.Choice(['asyncio', 'multi_asyncio']),
              default='asyncio',
              help='执行器类型')
@click.option('--concurrency', '-c', type=int, default=4, help='并发数')
@click.option('--prefetch', '-p', type=int, default=100, help='预取倍数')
@click.option('--reload', '-r', is_flag=True, help='自动重载')
@click.option('--redis-url', default='redis://localhost:6379/0', help='Redis URL')
@click.option('--config', help='配置文件 (JSON格式)')
def worker(app_str, queues, executor, concurrency, prefetch, reload, redis_url, config):
    """启动任务处理 Worker
    
    示例:
    \b
      jettask worker main:app --queues async_queue
      jettask worker tasks.py:app --queues queue1,queue2,queue3
      jettask worker myapp.tasks:app --queues high,normal,low --executor multi_asyncio
    """
    
    # 如果提供了配置文件，从中加载配置
    if config:
        click.echo(f"Loading configuration from {config}")
        with open(config, 'r') as f:
            config_data = json.load(f)
        
        # 从配置文件读取参数（命令行参数优先）
        queues = queues or ','.join(config_data.get('queues', [])) if config_data.get('queues') else None
        executor = executor or config_data.get('executor', 'asyncio')
        concurrency = concurrency if concurrency != 4 else config_data.get('concurrency', 4)
        prefetch = prefetch if prefetch != 100 else config_data.get('prefetch', 100)
        reload = reload or config_data.get('reload', False)
        redis_url = redis_url if redis_url != 'redis://localhost:6379/0' else config_data.get('redis_url', 'redis://localhost:6379/0')
    
    # 解析 app_str (格式: module:app_name 或 file.py:app_name)
    module = None
    file = None
    app_name = 'app'
    
    if ':' in app_str:
        module_or_file, app_name = app_str.split(':', 1)
        
        # 判断是文件还是模块
        if module_or_file.endswith('.py') or '/' in module_or_file or os.path.exists(module_or_file):
            # 如果是文件路径（包含 / 或 .py 结尾，或者文件存在）
            file = module_or_file
            # 如果没有 .py 后缀，尝试添加
            if not file.endswith('.py') and os.path.exists(file + '.py'):
                file = file + '.py'
        else:
            # 否则作为模块名处理（如 myapp.tasks）
            module = module_or_file
    else:
        # 如果没有冒号，判断是文件还是模块
        if app_str.endswith('.py') or '/' in app_str or os.path.exists(app_str):
            file = app_str
            if not file.endswith('.py') and os.path.exists(file + '.py'):
                file = file + '.py'
        else:
            module = app_str
    
    # 解析队列列表（支持逗号分隔）
    queue_list = [q.strip() for q in queues.split(',') if q.strip()]
    
    if not queue_list:
        click.echo("Error: Must specify at least one queue with --queues", err=True)
        click.echo("  Example: --queues queue1,queue2,queue3", err=True)
        sys.exit(1)
    
    # 加载模块
    try:
        if file:
            click.echo(f"Loading tasks from file: {file}")
            loaded_module = load_module_from_path(file)
        else:
            click.echo(f"Loading tasks from module: {module}")
            loaded_module = importlib.import_module(module)
        
        # 查找 Jettask 实例
        app = find_jettask_app(loaded_module)
        
        if not app:
            # 如果自动查找失败，尝试使用指定的名称
            if hasattr(loaded_module, app_name):
                app = getattr(loaded_module, app_name)
            else:
                click.echo(f"Error: Cannot find Jettask instance '{app_name}' in the module", err=True)
                sys.exit(1)
        
        click.echo(f"Found Jettask app: {app}")
        
    except ImportError as e:
        click.echo(f"Error loading module: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)
    
    # 显示配置信息
    click.echo("=" * 60)
    click.echo("JetTask Worker Configuration")
    click.echo("=" * 60)
    click.echo(f"Redis URL:    {redis_url}")
    click.echo(f"Queues:       {', '.join(queue_list)}")
    click.echo(f"Executor:     {executor}")
    click.echo(f"Concurrency:  {concurrency}")
    click.echo(f"Prefetch:     {prefetch}")
    click.echo(f"Auto-reload:  {reload}")
    click.echo("=" * 60)
    
    # 启动 Worker
    try:
        click.echo(f"Starting {executor} worker...")
        app.start(
            execute_type=executor,
            queues=queue_list,
            concurrency=concurrency,
            prefetch_multiplier=prefetch,
            reload=reload
        )
    except KeyboardInterrupt:
        click.echo("\nShutting down worker...")
    except Exception as e:
        click.echo(f"Error starting worker: {e}", err=True)
        sys.exit(1)

@cli.command('webui-consumer')
@click.option('--redis-url', default='redis://localhost:6379/0', help='Redis URL')
@click.option('--pg-url', help='PostgreSQL URL')
def webui_consumer(redis_url, pg_url):
    """启动 Web UI 数据消费者（同步 Redis 数据到 PostgreSQL）"""
    click.echo("Starting WebUI Consumer...")
    
    import os
    if pg_url:
        os.environ['JETTASK_PG_URL'] = pg_url
    
    # 启动 pg_consumer
    from jettask.webui.pg_consumer import main as consumer_main
    consumer_main()

@cli.command()
def monitor():
    """启动系统监控器"""
    click.echo("Starting JetTask Monitor")
    from jettask.webui.run_monitor import main as monitor_main
    monitor_main()

@cli.command()
def init():
    """初始化数据库和配置"""
    click.echo("Initializing JetTask...")
    
    # 初始化数据库
    from jettask.webui.db_init import init_database
    click.echo("Initializing database...")
    init_database()
    
    click.echo("JetTask initialized successfully!")

@cli.command()
def status():
    """显示系统状态"""
    click.echo("JetTask System Status")
    click.echo("=" * 50)
    
    # 检查 Redis 连接
    try:
        import redis
        r = redis.Redis(host='localhost', port=6379, db=0)
        r.ping()
        click.echo("✓ Redis: Connected")
    except:
        click.echo("✗ Redis: Not connected")
    
    # 检查 PostgreSQL 连接
    try:
        import psycopg2
        conn = psycopg2.connect(
            dbname=os.getenv('JETTASK_PG_DB', 'jettask'),
            user=os.getenv('JETTASK_PG_USER', 'jettask'),
            password=os.getenv('JETTASK_PG_PASSWORD', '123456'),
            host=os.getenv('JETTASK_PG_HOST', 'localhost'),
            port=os.getenv('JETTASK_PG_PORT', '5432')
        )
        conn.close()
        click.echo("✓ PostgreSQL: Connected")
    except:
        click.echo("✗ PostgreSQL: Not connected")
    
    click.echo("=" * 50)

def main():
    """主入口函数"""
    try:
        cli()
    except KeyboardInterrupt:
        click.echo("\nOperation cancelled by user")
        sys.exit(0)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        sys.exit(1)

if __name__ == '__main__':
    main()


import os
from typing import Optional
from dataclasses import dataclass


@dataclass
class PostgreSQLConfig:
    """PostgreSQL数据库配置"""
    host: str = "localhost"
    port: int = 5432
    database: str = "jettask"
    user: str = "jettask"
    password: str = "123456"
    
    @classmethod
    def from_env(cls) -> "PostgreSQLConfig":
        """从环境变量中读取配置"""
        return cls(
            host=os.getenv("JETTASK_PG_HOST", "localhost"),
            port=int(os.getenv("JETTASK_PG_PORT", "5432")),
            database=os.getenv("JETTASK_PG_DATABASE", "jettask"),
            user=os.getenv("JETTASK_PG_USER", "jettask"),
            password=os.getenv("JETTASK_PG_PASSWORD", "123456")
        )
    
    @classmethod
    def from_url(cls, url: str) -> "PostgreSQLConfig":
        """从数据库URL中解析配置
        格式: postgresql://user:password@host:port/database
        """
        from urllib.parse import urlparse
        
        parsed = urlparse(url)
        return cls(
            host=parsed.hostname or "localhost",
            port=parsed.port or 5432,
            database=parsed.path.lstrip("/") if parsed.path else "jettask",
            user=parsed.username or "jettask",
            password=parsed.password or "123456"
        )
    
    @property
    def dsn(self) -> str:
        """返回asyncpg连接字符串"""
        return f"postgresql://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}"
    
    @property
    def psycopg_dsn(self) -> str:
        """返回psycopg连接字符串"""
        return f"host={self.host} port={self.port} dbname={self.database} user={self.user} password={self.password}"


@dataclass
class RedisConfig:
    """Redis配置"""
    host: str = "localhost"
    port: int = 6379
    db: int = 0
    password: Optional[str] = None
    
    @classmethod
    def from_env(cls) -> "RedisConfig":
        """从环境变量中读取配置"""
        return cls(
            host=os.getenv("JETTASK_REDIS_HOST", "localhost"),
            port=int(os.getenv("JETTASK_REDIS_PORT", "6379")),
            db=int(os.getenv("JETTASK_REDIS_DB", "0")),
            password=os.getenv("JETTASK_REDIS_PASSWORD")
        )


@dataclass
class WebUIConfig:
    """WebUI配置"""
    host: str = "0.0.0.0"
    port: int = 8000
    reload: bool = False
    postgres: PostgreSQLConfig = None
    redis: RedisConfig = None
    
    def __post_init__(self):
        if self.postgres is None:
            self.postgres = PostgreSQLConfig.from_env()
        if self.redis is None:
            self.redis = RedisConfig.from_env()


import os
import sys
import time
import json
import signal
import socket
import asyncio
import logging
import threading
import contextlib
import importlib
import multiprocessing
from typing import List
from collections import defaultdict, deque

import redis
from redis import asyncio as aioredis
from watchdog.observers import Observer

from .task import Task
from .event_pool import EventPool
from ..executors import AsyncioExecutor, MultiAsyncioExecutor
from ..monitoring import FileChangeHandler
from ..utils import gen_task_name

logger = logging.getLogger('app')

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

# 尝试导入性能优化库
try:
    import uvloop
    UVLOOP_AVAILABLE = True
    # 自动启用uvloop
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    logger.debug("Using uvloop for better performance")
except ImportError:
    UVLOOP_AVAILABLE = False

try:
    import ujson
    # 使用更快的JSON库
    json_loads = ujson.loads
    json_dumps = ujson.dumps
    logger.debug("Using ujson for better JSON performance")
except ImportError:
    json_loads = json.loads
    json_dumps = json.dumps

_on_app_finalizers = set()

# 全局连接池复用
_redis_pools = {}
_async_redis_pools = {}

def get_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建Redis连接池"""
    if redis_url not in _redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _redis_pools[redis_url] = redis.ConnectionPool.from_url(
            redis_url, 
            decode_responses=True,
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _redis_pools[redis_url]

def get_async_redis_pool(redis_url: str, max_connections: int = 200):
    """获取或创建异步Redis连接池"""
    if redis_url not in _async_redis_pools:
        # 构建socket keepalive选项，仅在Linux上使用
        socket_keepalive_options = {}
        if hasattr(socket, 'TCP_KEEPIDLE'):
            socket_keepalive_options[socket.TCP_KEEPIDLE] = 1
        if hasattr(socket, 'TCP_KEEPINTVL'):
            socket_keepalive_options[socket.TCP_KEEPINTVL] = 3
        if hasattr(socket, 'TCP_KEEPCNT'):
            socket_keepalive_options[socket.TCP_KEEPCNT] = 5
        
        _async_redis_pools[redis_url] = aioredis.ConnectionPool.from_url(
            redis_url,
            decode_responses=True,
            max_connections=max_connections,
            retry_on_timeout=True,
            retry_on_error=[ConnectionError, TimeoutError],
            socket_keepalive=True,
            socket_keepalive_options=socket_keepalive_options if socket_keepalive_options else None,
            health_check_interval=30,
            # 优化超时配置以处理高负载
            socket_connect_timeout=10,  # 增加连接超时时间
            socket_timeout=15,          # 增加读取超时时间，避免频繁超时
        )
    return _async_redis_pools[redis_url]


def connect_on_app_finalize(callback):
    """Connect callback to be called when any app is finalized."""
    _on_app_finalizers.add(callback)
    return callback


class Jettask(object):

    def __init__(self, redis_url: str = None, include: list = None, max_connections: int = 200, 
                 consumer_strategy: str = None, consumer_config: dict = None, tasks=None,
                 redis_prefix: str = None) -> None:
        self._tasks = tasks or {}
        self.asyncio = False
        self.include = include or []
        self.redis_url = redis_url
        self.max_connections = max_connections
        self.consumer_strategy = consumer_strategy
        self.consumer_config = consumer_config or {}
        
        # Redis prefix configuration
        self.redis_prefix = redis_prefix or "jettask"
        
        # Update prefixes with the configured prefix using colon namespace
        self.STATUS_PREFIX = f"{self.redis_prefix}:STATUS:"
        self.RESULT_PREFIX = f"{self.redis_prefix}:RESULT:"
        
        # 预编译常用操作，减少运行时开销
        self._json_loads = json_loads
        self._json_dumps = json_dumps
        self._status_prefix = self.STATUS_PREFIX
        self._result_prefix = self.RESULT_PREFIX
        
        # 注册清理处理器
        self._setup_cleanup_handlers()
    
    def _setup_cleanup_handlers(self):
        """设置清理处理器"""
        self._cleanup_done = False
        self._should_exit = False
        self._worker_started = False  # 标记worker是否启动过
        
        def signal_cleanup_handler(signum=None, frame=None):
            """信号处理器"""
            if self._cleanup_done:
                return
            # 只有启动过worker才需要打印清理信息
            if self._worker_started:
                logger.info("Received shutdown signal, cleaning up...")
            self.cleanup()
            if signum:
                # 设置标记表示需要退出
                self._should_exit = True
                # 对于多进程环境，不直接操作事件循环
                # 让执行器自己检测退出标志并优雅关闭
        
        def atexit_cleanup_handler():
            """atexit处理器"""
            if self._cleanup_done:
                return
            # atexit时不重复打印日志，静默清理
            self.cleanup()
        
        # 注册信号处理器
        if hasattr(signal, 'SIGTERM'):
            signal.signal(signal.SIGTERM, signal_cleanup_handler)
        if hasattr(signal, 'SIGINT'):
            signal.signal(signal.SIGINT, signal_cleanup_handler)
        
        # 注册atexit处理器
        import atexit
        atexit.register(atexit_cleanup_handler)
    
    def cleanup(self):
        """清理应用资源"""
        if self._cleanup_done:
            return
        
        self._cleanup_done = True
        
        # 只有真正启动过worker才打印日志
        if self._worker_started:
            logger.info("Cleaning up Jettask resources...")
            
            # 清理EventPool
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()
            
            logger.info("Jettask cleanup completed")
        else:
            # 如果只是实例化但没有启动，静默清理
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()
            logger.debug("Jettask instance cleanup (no worker started)")
    
    @property
    def consumer_manager(self):
        """获取消费者管理器"""
        return self.ep.consumer_manager if hasattr(self.ep, 'consumer_manager') else None

    @property
    def async_redis(self):
        """优化：复用连接池"""
        name = "_async_redis"
        if hasattr(self, name):
            return getattr(self, name)
        
        pool = get_async_redis_pool(self.redis_url, self.max_connections)
        async_redis = aioredis.StrictRedis(connection_pool=pool)
        setattr(self, name, async_redis)
        return async_redis

    @property
    def redis(self):
        """优化：复用连接池"""
        name = "_redis"
        if hasattr(self, name):
            return getattr(self, name)
            
        pool = get_redis_pool(self.redis_url, self.max_connections)
        redis_cli = redis.StrictRedis(connection_pool=pool)
        setattr(self, name, redis_cli)
        return redis_cli

    @property
    def ep(self):
        name = "_ep"
        if hasattr(self, name):
            ep = getattr(self, name)
        else:
            # 传递redis_prefix到consumer_config
            consumer_config = self.consumer_config.copy() if self.consumer_config else {}
            consumer_config['redis_prefix'] = self.redis_prefix
            
            ep = EventPool(
                self.redis, 
                self.async_redis, 
                redis_url=self.redis_url,
                consumer_strategy=self.consumer_strategy,
                consumer_config=consumer_config,
                redis_prefix=self.redis_prefix
            )
            setattr(self, name, ep)
        return ep

    def clear(self):
        if hasattr(self, "process"):
            delattr(self, "process")
        if hasattr(self, "_ep"):
            delattr(self, "_ep")

    def get_task_by_name(self, name: str) -> Task:
        return self._tasks.get(name)

    def include_module(self, modules: list):
        self.include += modules

    def _task_from_fun(
        self, fun, name=None, base=None, queue=None, bind=False, **options
    ) -> Task:
        name = name or gen_task_name(fun.__name__, fun.__module__)
        base = base or Task
        if name not in self._tasks:
            run = staticmethod(fun)
            task: Task = type(
                fun.__name__,
                (base,),
                dict(
                    {
                        "app": self,
                        "name": name,
                        "run": run,
                        "queue": queue,
                        "bind": bind,
                        "_decorated": True,
                        "__doc__": fun.__doc__,
                        "__module__": fun.__module__,
                        "__annotations__": fun.__annotations__,
                        "__wrapped__": run,
                    },
                    **options,
                ),
            )()
            task.bind_app(self)
            with contextlib.suppress(AttributeError):
                task.__qualname__ = fun.__qualname__
            self._tasks[task.name] = task
        else:
            task = self._tasks[name]
        return task

    def task(
        self,
        name: str = None,
        queue: str = None,
        base: Task = None,
        *args,
        **kwargs,
    ):
        def _create_task_cls(fun):
            return self._task_from_fun(fun, name, base, queue, *args, **kwargs)

        return _create_task_cls

    def _mount_module(self):
        for module in self.include:
            module = importlib.import_module(module)
            for attr_name in dir(module):
                obj = getattr(module, attr_name)
                if hasattr(obj, "app"):
                    self._tasks.update(getattr(obj, "app")._tasks)

    def _validate_tasks_for_executor(self, execute_type: str, queues: List[str]):
        """验证任务类型是否与执行器兼容"""
        if execute_type in ["asyncio", "multi_asyncio"]:
            return  # AsyncIO和MultiAsyncio可以处理异步任务
        
        # 只有Thread执行器不能处理异步任务
        incompatible_tasks = []
        for task_name, task in self._tasks.items():
            # 检查任务是否属于指定队列
            if task.queue not in queues:
                continue
                
            # 检查是否是异步任务
            if asyncio.iscoroutinefunction(task.run):
                incompatible_tasks.append({
                    'name': task_name,
                    'queue': task.queue,
                    'type': 'async'
                })
        
        if incompatible_tasks:
            error_msg = f"\n错误：{execute_type} 执行器不能处理异步任务！\n"
            error_msg += "发现以下异步任务：\n"
            for task in incompatible_tasks:
                error_msg += f"  - {task['name']} (队列: {task['queue']})\n"
            error_msg += f"\n解决方案：\n"
            error_msg += f"1. 使用 asyncio 或 process 执行器\n"
            error_msg += f"2. 或者将这些任务改为同步函数（去掉 async/await）\n"
            error_msg += f"3. 或者将这些任务的队列从监听列表中移除\n"
            raise ValueError(error_msg)
    
    def _start(
        self,
        execute_type: str = "asyncio",
        queues: List[str] = None,
        concurrency: int = 1,
        prefetch_multiplier: int = 1,
        **kwargs
    ):
        # 设置默认队列
        if not queues:
            queues = [self.redis_prefix]
        
        self.ep.queues = queues
        self.ep.init_routing()
        self._mount_module()
        # 验证任务兼容性 
        self._validate_tasks_for_executor(execute_type, queues)
        
        event_queue = deque()
        
        # 创建消费者组
        try:
            self.ep.create_group()
        except Exception as e:
            logger.warning(f"创建消费者组时出错: {e}")
            # 继续执行，listening_event会自动处理
        
        # 根据执行器类型创建对应的执行器
        if execute_type == "asyncio":
            # 对于asyncio执行器，使用asyncio.Queue
            async_event_queue = asyncio.Queue()
            
            async def run_asyncio_executor():
                # 启动异步事件监听
                asyncio.create_task(self.ep.listening_event(async_event_queue, prefetch_multiplier))
                # 创建并运行执行器
                executor = AsyncioExecutor(async_event_queue, self, concurrency)
                await executor.loop()
            
            # try:
            loop = asyncio.get_event_loop()
            # except RuntimeError:
            #     # 如果当前线程没有事件循环，创建一个新的
            #     loop = asyncio.new_event_loop()
            #     asyncio.set_event_loop(loop)
            
            try:
                loop.run_until_complete(run_asyncio_executor())
            except RuntimeError as e:
                if "Event loop stopped" in str(e):
                    logger.info("Event loop stopped, shutting down gracefully")
                else:
                    raise
        elif execute_type == "multi_asyncio":
            # multi_asyncio在每个子进程中会启动自己的监听器
            executor = MultiAsyncioExecutor(event_queue, self, concurrency)
            executor.prefetch_multiplier = prefetch_multiplier
            
            # 设置信号处理器以正确响应Ctrl+C
            def multi_asyncio_signal_handler(signum, _frame):
                logger.info(f"Multi-asyncio mode received signal {signum}")
                executor._main_received_signal = True
                executor.shutdown_event.set()
                # 强制退出主循环
                raise KeyboardInterrupt()
            
            signal.signal(signal.SIGINT, multi_asyncio_signal_handler)
            signal.signal(signal.SIGTERM, multi_asyncio_signal_handler)
            
            try:
                executor.loop()
            except KeyboardInterrupt:
                logger.info("Multi-asyncio mode interrupted")
            finally:
                executor.shutdown()
        else:
            raise ValueError(f"不支持的执行器类型：{execute_type}，仅支持 'asyncio' 和 'multi_asyncio'")

    def _run_subprocess(self, *args, **kwargs):
        logger.info("Started Worker Process")
        process = multiprocessing.Process(target=self._start, args=args, kwargs=kwargs)
        process.start()
        return process

    def start(
        self,
        execute_type: str = "asyncio",
        queues: List[str] = None,
        concurrency: int = 1,
        prefetch_multiplier: int = 1,
        reload: bool = False,
    ):
        # 标记worker已启动
        self._worker_started = True
        
        if execute_type == "multi_asyncio" and self.consumer_strategy == "pod":
            raise ValueError("multi_asyncio模式下无法使用pod策略")
        self.process = self._run_subprocess(
            execute_type=execute_type,
            queues=queues,
            concurrency=concurrency,
            prefetch_multiplier=prefetch_multiplier,
        )
        if reload:
            event_handler = FileChangeHandler(
                self,
                execute_type=execute_type,
                queues=queues,
                concurrency=concurrency,
                prefetch_multiplier=prefetch_multiplier,
            )
            observer = Observer()
            observer.schedule(event_handler, ".", recursive=True)
            observer.start()
        # 使用事件来等待，而不是无限循环
        try:
            while not self._should_exit:
                time.sleep(0.1)  # 短暂睡眠，快速响应退出信号
        except KeyboardInterrupt:
            logger.info("Received keyboard interrupt")
            self.cleanup()
            if self.process and self.process.is_alive():
                self.process.terminate()
                self.process.join(timeout=2)
                if self.process.is_alive():
                    logger.warning("Process did not terminate, killing...")
                    self.process.kill()

    def bulk_write(self, tasks: list, asyncio: bool = False):
        """优化批量写入性能"""
        if not tasks:
            raise ValueError("tasks 参数不能为空！")
            
        # 优化：减少字典操作开销
        task_mapping = {}
        for task in tasks:
            queue = task["queue"]
            if queue not in task_mapping:
                task_mapping[queue] = []
            task_mapping[queue].append(task)
            
        if asyncio:
            return self._bulk_write(task_mapping)
        
        # 优化：批量处理减少网络往返
        event_ids = []
        
        for queue, queue_tasks in task_mapping.items():
            queue_event_ids = self.ep.batch_send_event(queue, queue_tasks)
            event_ids.extend(queue_event_ids)
            
            # 使用新的Hash方式批量设置初始状态
            current_time = str(time.time())
            pipeline = self.redis.pipeline()
            for event_id in queue_event_ids:
                key = f"{self.redis_prefix}:TASK:{event_id}"
                pipeline.hset(key, mapping={
                    "status": "pending",
                    "created_at": current_time
                })
                pipeline.expire(key, 3600)
        
        # 执行pipeline
        if event_ids:
            pipeline.execute()
            
        return event_ids

    async def _bulk_write(self, task_mapping):
        """异步批量写入优化"""
        event_ids = []
        
        for queue, tasks in task_mapping.items():
            queue_event_ids = await self.ep.batch_send_event(queue, tasks, asyncio=True)
            event_ids.extend(queue_event_ids)
            
            # 使用新的Hash方式批量设置初始状态
            current_time = str(time.time())
            pipeline = self.async_redis.pipeline()
            for event_id in queue_event_ids:
                key = f"{self.redis_prefix}:TASK:{event_id}"
                pipeline.hset(key, mapping={
                    "status": "pending",
                    "created_at": current_time
                })
                pipeline.expire(key, 3600)
        
        # 执行pipeline
        if event_ids:
            await pipeline.execute()
            
        return event_ids

    def get_task_status(self, event_id: str, asyncio: bool = False):
        """优化状态获取"""
        client = self.get_redis_client(asyncio)
        key = f"{self._status_prefix}{event_id}"
        ret = client.get(key)
        
        if asyncio:
            return self._get_task_status(ret)
        else:
            return self._json_loads(ret) if ret else None

    async def _get_task_status(self, ret):
        s = await ret
        return self._json_loads(s) if s else None

    def set_task_status(self, event_id: str, status: str, asyncio: bool = False):
        """优化状态设置"""
        client = self.get_redis_client(asyncio)
        key = f"{self._status_prefix}{event_id}"
        # Store status as a JSON object instead of just a string
        status_data = {
            "status": status,
            "updated_at": int(time.time() * 1000)
        }
        return client.set(key, self._json_dumps(status_data), ex=3600)

    def set_task_status_by_batch(self, mapping: dict, asyncio: bool = False):
        """优化批量状态设置"""
        client = self.get_redis_client(asyncio)
        
        # 预构建键值对，减少循环中的字符串操作
        current_time = int(time.time() * 1000)
        redis_mapping = {
            f"{self._status_prefix}{event_id}": self._json_dumps({
                "status": status,
                "updated_at": current_time
            })
            for event_id, status in mapping.items()
        }
        
        return client.mset(redis_mapping)

    def del_task_status(self, event_id: str, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        return client.delete(f"{self._status_prefix}{event_id}")

    def get_redis_client(self, asyncio: bool = False):
        return self.async_redis if asyncio else self.redis

    def set_data(
        self, event_id: str, result: str, ex: int = 3600, asyncio: bool = False
    ):
        """优化数据设置"""
        client = self.get_redis_client(asyncio)
        key = f"{self._result_prefix}{event_id}"
        return client.set(key, value=result, ex=ex)
    
    async def get_and_delayed_deletion(self, key: str, ex: int):
        client = self.get_redis_client(True)
        result = await client.get(key)
        await client.expire(key, time=ex)
        return result
    
    def get_result(self, event_id: str, delete: bool = False, asyncio: bool = False, delayed_deletion_ex: int = None):
        """优化结果获取"""
        client = self.get_redis_client(asyncio)
        key = f"{self._result_prefix}{event_id}"
        
        if delayed_deletion_ex is not None:
            return self.get_and_delayed_deletion(key, delayed_deletion_ex)
        elif delete:
            return client.getdel(key)
        else:
            return client.get(key)


import ujson
import time
from dataclasses import dataclass
from typing import Any, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from .app import Jettask


@dataclass
class ExecuteResponse:
    delay: Optional[float] = None 
    urgent_retry: bool = False 
    reject: bool = False
    retry_time: Optional[float] = None


class Request:
    id: str = None
    name: str = None
    app: "Jettask" = None

    def __init__(self, *args, **kwargs) -> None:
        self._update(*args, **kwargs)

    def _update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)


class Task:
    _app: "Jettask" = None
    name: str = None
    queue: str = None
    bind: bool = False
    trigger_time: float = None

    def __call__(self, event_id: str, trigger_time: float, *args: Any, **kwds: Any) -> Any:
        if self.bind:
            request = Request(id=event_id, name=self.name, app=self._app, trigger_time=trigger_time)
            args = [request] + list(args)
        return self.run(*args, **kwds)

    def run(self, *args, **kwargs):
        """The body of the task executed by workers."""
        raise NotImplementedError("Tasks must define the run method.")

    @classmethod
    def bind_app(cls, app):
        cls._app = app

    def apply_async(
        self,
        args: tuple = None,
        kwargs: dict = None,
        queue: str = None,
        at_once: bool = True,
        asyncio: bool = False,
        routing: dict = None,
    ):
        queue = queue or self.queue
        message = {
            "queue": queue,
            "name": self.name,
            "args": ujson.dumps(args or ()),
            "kwargs": ujson.dumps(kwargs or {}),
            'trigger_time': time.time()
        }
        if routing:
           message['routing'] = ujson.dumps(routing or {})
        if not at_once:
            return message
        if asyncio:
            return self._send_task(queue, message)
        else:
            return self.send_task(queue, message)

    def on_before(self, event_id, pedding_count, args, kwargs) -> ExecuteResponse:
        return ExecuteResponse()

    def on_end(self, event_id, pedding_count, args, kwargs, result) -> ExecuteResponse:
        return ExecuteResponse()

    def on_success(self, event_id, args, kwargs, result) -> ExecuteResponse:
        return ExecuteResponse()

    def send_task(self, queue, message):
        event_id = self._app.ep.send_event(queue, message, False)
        # 使用新的Hash方式设置初始状态
        key = f"{self._app.ep.redis_prefix or 'jettask'}:TASK:{event_id}"
        self._app.redis.hset(key, mapping={
            "status": "pending",
            "created_at": str(time.time())
        })
        self._app.redis.expire(key, 3600)
        return event_id

    async def _send_task(self, queue, message):
        event_id = await self._app.ep.send_event(queue, message, True)
        # 使用新的Hash方式设置初始状态
        key = f"{self._app.ep.redis_prefix or 'jettask'}:TASK:{event_id}"
        await self._app.async_redis.hset(key, mapping={
            "status": "pending",
            "created_at": str(time.time())
        })
        await self._app.async_redis.expire(key, 3600)
        return event_id

    def read_pending(
        self,
        queue: str = None,
        asyncio: bool = False,
    ):
        queue = queue or self.queue
        if asyncio:
            return self._get_pending(queue)
        return self._app.ep.read_pending(queue, queue)

    async def _get_pending(self, queue: str):
        return await self._app.ep.read_pending(queue, queue, asyncio=True)


import json
import time
import threading
import logging
import contextlib
import asyncio
from collections import defaultdict, deque, Counter
from typing import List, Optional, TYPE_CHECKING, Union

import redis
from redis import asyncio as aioredis


from ..utils.helpers import get_hostname
from .consumer_manager import ConsumerManager, ConsumerStrategy

logger = logging.getLogger('app')


class EventPool(object):
    STATE_MACHINE_NAME = "STATE_MACHINE"
    TIMEOUT = 60 * 5

    def __init__(
        self,
        redis_client: redis.StrictRedis,
        async_redis_client: aioredis.StrictRedis,
        queues: list = None,
        redis_url: str = None,
        consumer_strategy: str = None,
        consumer_config: dict = None,
        redis_prefix: str = None,
    ) -> None:
        self.redis_client = redis_client
        self.async_redis_client = async_redis_client
        self.queues = queues
        self._redis_url = redis_url or 'redis://localhost:6379/0'
        self.redis_prefix = redis_prefix or 'jettask'
        
        # 初始化消费者管理器
        strategy = ConsumerStrategy(consumer_strategy) if consumer_strategy else ConsumerStrategy.HEARTBEAT
        # 确保配置中包含队列信息
        manager_config = consumer_config or {}
        manager_config['queues'] = queues or []
        
        self.consumer_manager = ConsumerManager(
            redis_client=redis_client,
            strategy=strategy,
            config=manager_config
        )
        
        # 创建带前缀的队列名称映射
        self.prefixed_queues = {}
        
        self.solo_routing_tasks = {}
        self.solo_running_state = {}
        self.solo_urgent_retry = {}
        self.batch_routing_tasks = {}
        self.task_scheduler = {}
        self.running_task_state_mappings = {}
        self.delay_tasks = []
        self.solo_agg_task = {}
        self.rlock = threading.RLock()
        self._claimed_message_ids = set()  # 跟踪已认领的消息ID，防止重复处理
    
    def _put_task(self, event_queue: Union[deque, asyncio.Queue], task, urgent: bool = False):
        """统一的任务放入方法"""
        # 如果是deque，使用原有逻辑
        if isinstance(event_queue, deque):
            if urgent:
                event_queue.appendleft(task)
            else:
                event_queue.append(task)
        # 如果是asyncio.Queue，则暂时只能按顺序放入（Queue不支持优先级）
        elif isinstance(event_queue, asyncio.Queue):
            # 对于asyncio.Queue，我们需要在async上下文中操作
            # 这里先保留接口，具体实现在async方法中
            pass
    
    async def _async_put_task(self, event_queue: asyncio.Queue, task, urgent: bool = False):
        """异步任务放入方法"""
        await event_queue.put(task)

    def init_routing(self):
        for queue in self.queues:
            self.solo_agg_task[queue] = defaultdict(list)
            self.solo_routing_tasks[queue] = defaultdict(list)
            self.solo_running_state[queue]  = defaultdict(bool)
            self.batch_routing_tasks[queue]  = defaultdict(list)
            self.task_scheduler[queue] = defaultdict(int)
            self.running_task_state_mappings[queue] = defaultdict(dict)
            
    def get_prefixed_queue_name(self, queue: str) -> str:
        """为队列名称添加前缀"""
        return f"{self.redis_prefix}:QUEUE:{queue}"
    
    def get_redis_client(self, asyncio: bool = False):
        return self.async_redis_client if asyncio else self.redis_client

    def create_group(self):
        for queue in self.queues:
            prefixed_queue = self.get_prefixed_queue_name(queue)
            with contextlib.suppress(Exception):
                self.redis_client.xgroup_create(
                    name=prefixed_queue,
                    groupname=prefixed_queue,
                    id="0",
                    mkstream=True,
                )

    def send_event(self, queue, message: dict, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        prefixed_queue = self.get_prefixed_queue_name(queue)
        try:
            return client.xadd(prefixed_queue, message, maxlen=3000)
        except redis.exceptions.ResponseError as e:
            # 如果队列不存在，创建它
            if "ERR" in str(e):
                logger.warning(f'队列 {prefixed_queue} 不存在，正在创建...')
                try:
                    # 先创建队列
                    event_id = client.xadd(prefixed_queue, message, maxlen=3000)
                    # 再创建消费者组
                    with contextlib.suppress(Exception):
                        client.xgroup_create(
                            name=prefixed_queue,
                            groupname=prefixed_queue,
                            id="0"
                        )
                    return event_id
                except Exception as create_error:
                    logger.error(f'创建队列失败: {create_error}')
                    raise
            else:
                raise

    def batch_send_event(self, queue, messages: List[dict], asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        pipe = client.pipeline()
        prefixed_queue = self.get_prefixed_queue_name(queue)
        if asyncio:
            return self._batch_send_event(prefixed_queue, messages, pipe)
        for message in messages:
            pipe.xadd(prefixed_queue, message)
        return pipe.execute()

    async def _batch_send_event(self, prefixed_queue, messages: List[dict], pipe):
        for message in messages:
            await pipe.xadd(prefixed_queue, message)
        return await pipe.execute()
    
    def is_urgent(self, routing_key):
        is_urgent = self.solo_urgent_retry.get(routing_key, False)
        if is_urgent == True:
            del self.solo_urgent_retry[routing_key]
        return is_urgent
    
    @classmethod
    def separate_by_key(cls, lst):
        groups = {}
        for item in lst:
            key = item[0]['routing_key']
            if key not in groups:
                groups[key] = []
            groups[key].append(item)
        result = []
        group_values = list(groups.values())
        while True:
            exists_data = False
            for values in group_values:
                try:
                    result.append(values.pop(0))
                    exists_data = True
                except:
                    pass
            if not exists_data:
                break
        return result
    
    async def async_check_solo_agg_tasks(self, event_queue: asyncio.Queue):
        """异步版本的聚合任务检查"""
        last_solo_running_state = defaultdict(dict)
        last_wait_time = defaultdict(int)
        queue_batch_tasks = defaultdict(list)
        left_queue_batch_tasks = defaultdict(list)
        
        while True:
            has_work = False
            current_time = time.time()
            
            for queue in self.queues:
                for agg_key, tasks in self.solo_agg_task[queue].items():
                    if not tasks:
                        continue
                        
                    has_work = True
                    need_del_index = []
                    need_lock_routing_keys = []
                    sort_by_tasks = self.separate_by_key(tasks)
                    max_wait_time = 5
                    max_records = 3
                    
                    for index, (routing, task) in enumerate(sort_by_tasks):
                        routing_key = routing['routing_key']
                        max_records = routing.get('max_records', 1)
                        max_wait_time = routing.get('max_wait_time', 0)
                        
                        with self.rlock:
                            if self.solo_running_state[queue].get(routing_key, 0) > 0:
                                continue
                                
                        if len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records:
                            break 
                            
                        task["routing"] = routing

                        if self.is_urgent(routing_key):
                            left_queue_batch_tasks[queue].append(task)
                        else:
                            queue_batch_tasks[queue].append(task)
                        need_lock_routing_keys.append(routing_key)
                        need_del_index.append(index)

                    for routing_key, count in Counter(need_lock_routing_keys).items():
                        with self.rlock:
                            self.solo_running_state[queue][routing_key] = count
                            
                    if last_solo_running_state[queue] != self.solo_running_state[queue]:
                        last_solo_running_state[queue] = self.solo_running_state[queue].copy()
                        
                    tasks = [task for index, task in enumerate(sort_by_tasks) if index not in need_del_index]
                    self.solo_agg_task[queue][agg_key] = tasks
                    
                    if (len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records or 
                        (last_wait_time[queue] and last_wait_time[queue] < current_time - max_wait_time)):
                        for task in queue_batch_tasks[queue]:
                            await self._async_put_task(event_queue, task)
                        for task in left_queue_batch_tasks[queue]:
                            await self._async_put_task(event_queue, task)    
                        queue_batch_tasks[queue] = []
                        left_queue_batch_tasks[queue] = []
                        last_wait_time[queue] = 0
                    elif last_wait_time[queue] == 0:
                        last_wait_time[queue] = current_time
                        
            # 优化：根据是否有工作决定休眠时间
            if has_work:
                await asyncio.sleep(0.001)  # 有工作时极短休眠
            else:
                await asyncio.sleep(0.01)   # 无工作时短暂休眠
    
    def check_solo_agg_tasks(self, event_queue: Union[deque, asyncio.Queue]):
        """优化聚合任务检查，减少休眠时间"""
        last_solo_running_state = defaultdict(dict)
        last_wait_time = defaultdict(int)
        queue_batch_tasks = defaultdict(list)
        left_queue_batch_tasks = defaultdict(list)
        
        while True:
            has_work = False
            current_time = time.time()
            
            for queue in self.queues:
                for agg_key, tasks in self.solo_agg_task[queue].items():
                    if not tasks:
                        continue
                        
                    has_work = True
                    need_del_index = []
                    need_lock_routing_keys = []
                    sort_by_tasks = self.separate_by_key(tasks)
                    max_wait_time = 5
                    max_records = 3
                    
                    for index, (routing, task) in enumerate(sort_by_tasks):
                        routing_key = routing['routing_key']
                        max_records = routing.get('max_records', 1)
                        max_wait_time = routing.get('max_wait_time', 0)
                        
                        with self.rlock:
                            if self.solo_running_state[queue].get(routing_key, 0) > 0:
                                continue
                                
                        if len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records:
                            break 
                            
                        task["routing"] = routing

                        if self.is_urgent(routing_key):
                            left_queue_batch_tasks[queue].append(task)
                        else:
                            queue_batch_tasks[queue].append(task)
                        need_lock_routing_keys.append(routing_key)
                        need_del_index.append(index)

                    for routing_key, count in Counter(need_lock_routing_keys).items():
                        with self.rlock:
                            self.solo_running_state[queue][routing_key] = count
                            
                    if last_solo_running_state[queue] != self.solo_running_state[queue]:
                        last_solo_running_state[queue] = self.solo_running_state[queue].copy()
                        
                    tasks = [task for index, task in enumerate(sort_by_tasks) if index not in need_del_index]
                    self.solo_agg_task[queue][agg_key] = tasks
                    
                    if (len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records or 
                        (last_wait_time[queue] and last_wait_time[queue] < current_time - max_wait_time)):
                        for task in queue_batch_tasks[queue]:
                            self._put_task(event_queue, task)
                        for task in left_queue_batch_tasks[queue]:
                            self._put_task(event_queue, task, urgent=True)    
                        queue_batch_tasks[queue] = []
                        left_queue_batch_tasks[queue] = []
                        last_wait_time[queue] = 0
                    elif last_wait_time[queue] == 0:
                        last_wait_time[queue] = current_time
                        
            # 优化：根据是否有工作决定休眠时间
            if has_work:
                time.sleep(0.001)  # 有工作时极短休眠
            else:
                time.sleep(0.01)   # 无工作时短暂休眠
    
    def check_sole_tasks(self, event_queue: Union[deque, asyncio.Queue]):
        agg_task_mappings = {queue:  defaultdict(list) for queue in self.queues}
        agg_wait_task_mappings = {queue:  defaultdict(float) for queue in self.queues}
        task_max_wait_time_mapping = {}
        make_up_for_index_mappings = {queue:  defaultdict(int) for queue in self.queues} 
        while True:
            put_count = 0
            for queue in self.queues:
                agg_task = agg_task_mappings[queue]
                for routing_key, tasks in self.solo_routing_tasks[queue].items():
                    schedule_time = self.task_scheduler[queue][routing_key]
                    if tasks:
                        for task in tasks:
                            prev_routing = task[0]
                            if agg_key:= prev_routing.get('agg_key'):
                                if not self.running_task_state_mappings[queue][agg_key]:
                                    self.solo_running_state[queue][routing_key] = False
                                    break 
                    if (
                        schedule_time <= time.time()
                        and self.solo_running_state[queue][routing_key] == False
                    ) :
                            try:
                                routing, task = tasks.pop(0)
                            except IndexError:
                                continue
                            task["routing"] = routing
                            
                            agg_key = routing.get('agg_key')
                            if agg_key is not None:
                                start_time = agg_wait_task_mappings[queue][agg_key]
                                if not start_time:
                                    agg_wait_task_mappings[queue][agg_key] = time.time()
                                    start_time = agg_wait_task_mappings[queue][agg_key]
                                agg_task[agg_key].append(task)
                                max_wait_time = routing.get('max_wait_time', 3)
                                task_max_wait_time_mapping[agg_key] = max_wait_time
                                if len(agg_task[agg_key])>=routing.get('max_records', 100) or time.time()-start_time>=max_wait_time:
                                    logger.info(f'{agg_key=} {len(agg_task[agg_key])} 已满，准备发车！{routing.get("max_records", 100)} {time.time()-start_time} {max_wait_time}')
                                    for task in agg_task[agg_key]:
                                        task['routing']['version'] = 1
                                        self.running_task_state_mappings[queue][agg_key][task['event_id']] = time.time()
                                        self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                                    agg_task[agg_key] = []
                                    make_up_for_index_mappings[queue][agg_key] = 0 
                                    agg_wait_task_mappings[queue][agg_key] = 0
                            else:
                                self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                            self.solo_running_state[queue][routing_key] = True
                            put_count += 1
                for agg_key in agg_task.keys():
                    if not agg_task[agg_key]:
                        continue
                    start_time = agg_wait_task_mappings[queue][agg_key]
                    max_wait_time = task_max_wait_time_mapping[agg_key]
                    if make_up_for_index_mappings[queue][agg_key]>= len(agg_task[agg_key])-1:
                        make_up_for_index_mappings[queue][agg_key] = 0
                    routing = agg_task[agg_key][make_up_for_index_mappings[queue][agg_key]]['routing']
                    routing_key = routing['routing_key']
                    self.solo_running_state[queue][routing_key] = False
                    make_up_for_index_mappings[queue][agg_key] += 1
                    if time.time()-start_time>=max_wait_time:
                        logger.info(f'{agg_key=} {len(agg_task[agg_key])}被迫发车！ {time.time()-start_time} {max_wait_time}')
                        for task in agg_task[agg_key]:
                            task['routing']['version'] = 1
                            self.running_task_state_mappings[queue][agg_key][task['event_id']] = time.time()
                            self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                        agg_task[agg_key] = []
                        make_up_for_index_mappings[queue][agg_key] = 0
                        agg_wait_task_mappings[queue][agg_key] = 0
            # 优化：根据处理任务数量动态调整休眠时间
            if not put_count:
                time.sleep(0.001)
            elif put_count < 5:
                time.sleep(0.0005)  # 少量任务时极短休眠
                
    def check_batch_tasks(self, event_queue: Union[deque, asyncio.Queue]):
        agg_task_mappings = {queue:  defaultdict(list) for queue in self.queues}
        agg_wait_task_mappings = {queue:  defaultdict(float) for queue in self.queues}
        while True:
            for queue in self.queues:
                agg_task = agg_task_mappings[queue]
                for _, tasks in self.batch_routing_tasks[queue].items():
                    try:
                        metadata, task = tasks.pop(0)
                    except IndexError:
                        continue
                    agg_key = metadata['agg_key']
                    start_time = agg_wait_task_mappings[queue][agg_key]
                    if not start_time:
                        agg_wait_task_mappings[queue][agg_key] = time.time()
                        start_time = agg_wait_task_mappings[queue][agg_key]
                    max_wait_time = metadata['max_wait_time']
                    max_records = metadata['max_records']
                    agg_task[agg_key].append(task)
                    if len(agg_task[agg_key])>=max_records or time.time()-start_time>=max_wait_time:
                        self._put_task(event_queue, {
                            'queue': queue,
                            'task_type': 'batch',
                            'event_ids': [item['event_id'] for item in agg_task[agg_key]],
                            'event_data_list': [item['event_data'] for item in agg_task[agg_key]],
                            'name': agg_task[agg_key][0]['event_data']['name'],
                        })
                        agg_task[agg_key] = []
                        agg_wait_task_mappings[queue][agg_key] = time.time()
                for agg_key in agg_task.keys():
                    if not agg_task[agg_key]:
                        continue
                    start_time = agg_wait_task_mappings[queue][agg_key]
                    max_wait_time = json.loads(agg_task[agg_key][0]['event_data']['routing'])['max_wait_time']
                    if  time.time()-start_time>=max_wait_time:
                        self._put_task(event_queue, {
                            'queue': queue,
                            'task_type': 'batch',
                            'event_ids': [item['event_id'] for item in agg_task[agg_key]],
                            'event_data_list': [item['event_data'] for item in agg_task[agg_key]],
                            'name': agg_task[agg_key][0]['event_data']['name'],
                        })
                        agg_task[agg_key] = []
                        agg_wait_task_mappings[queue][agg_key] = time.time()
            # 优化：减少休眠时间
            time.sleep(0.001)

    def check_delay_tasks(self, event_queue):
        while True:
            put_count = 0
            need_del_index = []
            for i in range(len(self.delay_tasks)):
                schedule_time = self.delay_tasks[i][0]
                task = self.delay_tasks[i][1]
                if schedule_time <= time.time():
                    try:
                        self._put_task(event_queue, task)
                        need_del_index.append(i)
                        put_count += 1
                    except IndexError:
                        pass
            for i in need_del_index:
                del self.delay_tasks[i]
            if not put_count:          
                time.sleep(1)

    def _handle_redis_error(self, error: Exception, consecutive_errors: int, queue: str = None) -> tuple[bool, int]:
        """处理Redis错误的通用方法
        返回: (should_recreate_connection, new_consecutive_errors)
        """
        if isinstance(error, redis.exceptions.ConnectionError):
            logger.error(f'Redis连接错误: {error}')
            consecutive_errors += 1
            if consecutive_errors >= 5:
                logger.error(f'连续连接失败{consecutive_errors}次，重新创建连接')
                return True, 0
            return False, consecutive_errors
            
        elif isinstance(error, redis.exceptions.ResponseError):
            if "NOGROUP" in str(error) and queue:
                logger.warning(f'队列 {queue} 或消费者组不存在')
                return False, consecutive_errors
            else:
                logger.error(f'Redis错误: {error}')
                consecutive_errors += 1
                return False, consecutive_errors
        else:
            logger.error(f'意外错误: {error}')
            consecutive_errors += 1
            return False, consecutive_errors

    def _process_message_common(self, event_id: str, event_data: dict, queue: str, event_queue, is_async: bool = False, consumer_name: str = None):
        """通用的消息处理逻辑，供同步和异步版本使用"""
        # 检查消息是否已被认领，防止重复处理
        if event_id in self._claimed_message_ids:
            logger.debug(f"跳过已认领的消息 {event_id}")
            return event_id
        
        routing = event_data.get("routing")
        task_item = {
            "queue": queue,
            "event_id": event_id,
            "event_data": event_data,
            "consumer": consumer_name,  # 添加消费者信息
        }
        
        push_flag = True
        if routing:
            routing = json.loads(routing)
            if agg_key := routing.get('agg_key'):
                self.solo_agg_task[queue][agg_key].append(
                    [routing, task_item]
                )
                push_flag = False
        
        if push_flag:
            if is_async:
                # 这里不能直接await，需要返回一个标记
                return ('async_put', task_item)
            else:
                self._put_task(event_queue, task_item)
        
        return event_id

    async def listening_event(self, event_queue: asyncio.Queue, prefetch_multiplier: int = 1):
        async def listen_event_by_group(queue):
            check_backlog = True
            lastid = "0-0"
            consecutive_errors = 0
            max_consecutive_errors = 5

            # 启动异步任务处理solo_agg_tasks
            asyncio.create_task(self.async_check_solo_agg_tasks(event_queue))
            
            while True:
                if check_backlog:
                    myid = lastid
                else:
                    myid = ">"
                # 检查队列大小
                c_qsize = event_queue.qsize()
                
                if c_qsize >= max(prefetch_multiplier // 2, 1):
                    await asyncio.sleep(0.5)
                    continue
                logger.debug(
                    f"读取{queue}数据，当前堆积消息数：{c_qsize} {prefetch_multiplier-c_qsize=}"
                )
                try:
                    # 检查心跳是否超时
                    if self.consumer_manager.is_heartbeat_timeout():
                        logger.error(f"Heartbeat timeout detected for queue {queue}, shutting down...")
                        logger.error("Worker detected its own heartbeat timeout - likely network issue or being replaced")
                        logger.error("Initiating graceful shutdown to avoid conflicts with replacement worker")
                        
                        # 设置停止标志，触发优雅退出
                        self._stop_reading = True
                        
                        # 标记所有队列停止读取
                        for q in self.queues:
                            if q in self._queue_stop_flags:
                                self._queue_stop_flags[q] = True
                        
                        # 发送系统退出信号进行优雅关闭
                        import os
                        import signal
                        os.kill(os.getpid(), signal.SIGTERM)
                        return
                    
                    # 使用消费者管理器获取消费者名称
                    consumer_name = self.consumer_manager.get_consumer_name(queue)
                    # 计算要读取的消息数
                    count_to_read = max(1, prefetch_multiplier - c_qsize)
                    
                    prefixed_queue = self.get_prefixed_queue_name(queue)
                    messages = await self.async_redis_client.xreadgroup(
                        groupname=prefixed_queue,
                        consumername=consumer_name,
                        streams={prefixed_queue: myid},
                        count=count_to_read,
                        block=1000,  # 设置1秒超时而不是无限阻塞
                    )
                    # 成功读取，重置错误计数
                    consecutive_errors = 0
                    
                    if messages:
                        msg_count = len(messages[0][1]) if messages else 0
                        logger.info(f"Read {msg_count} messages from queue {queue}")
                        if msg_count > 0:
                            logger.info(f"Processing {msg_count} messages...")
                    
                except redis.exceptions.TimeoutError as e:
                    logger.warning(f'Redis读取超时: {e}，尝试重新连接...')
                    consecutive_errors += 1
                    if consecutive_errors >= max_consecutive_errors:
                        logger.error(f'连续超时{consecutive_errors}次，重新创建连接')
                        self._recreate_redis_connection()
                        consecutive_errors = 0
                    await asyncio.sleep(min(consecutive_errors * 0.5, 5))  # 指数退避
                    continue
                    
                except redis.exceptions.ConnectionError as e:
                    logger.error(f'Redis连接错误: {e}')
                    consecutive_errors += 1
                    if consecutive_errors >= max_consecutive_errors:
                        logger.error(f'连续连接失败{consecutive_errors}次，重新创建连接')
                        self._recreate_redis_connection()
                        consecutive_errors = 0
                    await asyncio.sleep(min(consecutive_errors, 10))  # 指数退避，最多10秒
                    continue
                    
                except redis.exceptions.ResponseError as e:
                    if "NOGROUP" in str(e):
                        # 队列或消费者组不存在，尝试重新创建
                        logger.warning(f'队列 {queue} 或消费者组不存在，尝试重新创建...')
                        try:
                            # 尝试创建消费者组 - 使用带前缀的队列名
                            prefixed_queue = self.get_prefixed_queue_name(queue)
                            await self.async_redis_client.xgroup_create(
                                name=prefixed_queue,
                                groupname=prefixed_queue,
                                id="0",
                                mkstream=True
                            )
                            logger.debug(f'成功重新创建队列 {prefixed_queue} 和消费者组')
                            check_backlog = True
                            lastid = "0-0"  # 重置ID，从头开始读取
                            consecutive_errors = 0
                            continue
                        except Exception as create_error:
                            logger.error(f'创建队列失败: {create_error}')
                            consecutive_errors += 1
                            await asyncio.sleep(min(consecutive_errors, 5))
                            continue
                    else:
                        logger.error(f'Redis错误: {e}')
                        consecutive_errors += 1
                        await asyncio.sleep(min(consecutive_errors, 5))
                        continue
                        
                except Exception as e:
                    logger.error(f'意外错误: {e}')
                    consecutive_errors += 1
                    await asyncio.sleep(min(consecutive_errors, 5))
                    continue
                
                # 检查messages是否为空
                if not messages:
                    check_backlog = False
                    continue
                    
                check_backlog = False if len(messages[0][1]) == 0 else True

                for message in messages:
                    for event in message[1]:
                        event_id = event[0]
                        event_data = event[1]
                        
                        # 使用通用的消息处理方法
                        result = self._process_message_common(event_id, event_data, queue, event_queue, is_async=True, consumer_name=consumer_name)
                        if isinstance(result, tuple) and result[0] == 'async_put':
                            # 异步放入队列
                            await self._async_put_task(event_queue, result[1])
                        lastid = event_id

        logger.info(f"Starting event listeners for queues: {self.queues}")
        # 使用asyncio创建并发任务
        tasks = []
        for queue in self.queues:
            logger.info(f"Starting listener task for queue: {queue}")
            task = asyncio.create_task(listen_event_by_group(queue))
            tasks.append(task)
        
        # 等待所有任务
        await asyncio.gather(*tasks)
    def read_pending(self, groupname: str, queue: str, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        prefixed_queue = self.get_prefixed_queue_name(queue)
        return client.xpending(prefixed_queue, groupname)

    def ack(self, queue, event_id, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        prefixed_queue = self.get_prefixed_queue_name(queue)
        result = client.xack(prefixed_queue, prefixed_queue, event_id)
        # 清理已认领的消息ID
        if event_id in self._claimed_message_ids:
            self._claimed_message_ids.remove(event_id)
        return result
    
    def _recreate_redis_connection(self):
        """重新创建Redis连接"""
        try:
            logger.info("开始重新创建Redis连接...")
            
            # 关闭现有连接
            if hasattr(self.redis_client, 'connection_pool'):
                try:
                    self.redis_client.connection_pool.disconnect()
                except:
                    pass
            
            if hasattr(self.async_redis_client, 'connection_pool'):
                try:
                    self.async_redis_client.connection_pool.disconnect()
                except:
                    pass
            
            # 重新创建连接池和客户端
            from ..core.app import get_redis_pool, get_async_redis_pool
            import redis
            from redis import asyncio as aioredis
            
            redis_url = self._redis_url
            
            # 重新创建同步连接
            pool = get_redis_pool(redis_url)
            new_redis_client = redis.StrictRedis(connection_pool=pool)
            
            # 重新创建异步连接
            async_pool = get_async_redis_pool(redis_url)
            new_async_redis_client = aioredis.StrictRedis(connection_pool=async_pool)
            
            # 测试新连接
            new_redis_client.ping()
            
            # 更新连接
            self.redis_client = new_redis_client
            self.async_redis_client = new_async_redis_client
            
            logger.info("Redis连接已成功重新创建")
            
        except Exception as e:
            logger.error(f"重新创建Redis连接失败: {e}")
            # 如果重新创建失败，尝试重置连接池
            try:
                if hasattr(self.redis_client, 'connection_pool'):
                    self.redis_client.connection_pool.reset()
                if hasattr(self.async_redis_client, 'connection_pool'):
                    self.async_redis_client.connection_pool.reset()
                logger.info("已重置现有连接池")
            except Exception as reset_error:
                logger.error(f"重置连接池失败: {reset_error}")
                
    def _safe_redis_operation(self, operation, *args, max_retries=3, **kwargs):
        """安全的Redis操作，带有重试机制"""
        for attempt in range(max_retries):
            try:
                return operation(*args, **kwargs)
            except (redis.exceptions.TimeoutError, redis.exceptions.ConnectionError) as e:
                if attempt == max_retries - 1:
                    logger.error(f"Redis操作失败，已重试{max_retries}次: {e}")
                    raise
                
                logger.warning(f"Redis操作失败，第{attempt + 1}次重试: {e}")
                if attempt == 0:  # 第一次失败时重新创建连接
                    self._recreate_redis_connection()
                time.sleep(min(2 ** attempt, 5))  # 指数退避，最多5秒
    
    def cleanup(self):
        """清理EventPool资源"""
        # 只有在有实际资源需要清理时才打印日志
        has_active_resources = False
        
        # 检查是否有活跃的消费者管理器
        if hasattr(self, 'consumer_manager') and self.consumer_manager:
            # 检查消费者管理器是否真的有活动
            if hasattr(self.consumer_manager, '_heartbeat_strategy'):
                strategy = self.consumer_manager._heartbeat_strategy
                if strategy and hasattr(strategy, 'consumer_id') and strategy.consumer_id:
                    has_active_resources = True
        
        if has_active_resources:
            logger.info("Cleaning up EventPool resources...")
            self.consumer_manager.cleanup()
            logger.info("EventPool cleanup completed")
        else:
            # 静默清理
            if hasattr(self, 'consumer_manager') and self.consumer_manager:
                self.consumer_manager.cleanup()


import time
import logging
from typing import TYPE_CHECKING
from watchdog.events import FileSystemEventHandler

if TYPE_CHECKING:
    from ..core.app import Jettask

logger = logging.getLogger('app')


class FileChangeHandler(FileSystemEventHandler):
    """监听文件变化的处理程序"""
    
    def __init__(self, app: "Jettask", **kwargs) -> None:
        super().__init__()
        self.app = app
        self.kwargs = kwargs
        self.last_modified = 0
        self.interval = 1  # 设置时间间隔为1秒

    def on_modified(self, event):
        current_time = time.time()
        if (
            event.src_path.endswith(".py")
            and current_time - self.last_modified > self.interval
        ):
            logging.warning(
                f'StatReload detected file change in "{event.src_path}". Reloading...'
            )
            try:
                self.app.process.terminate()
                self.app.process.join()
                self.app.clear()
                self.app.process = self.app._run_subprocess(**self.kwargs)
                self.last_modified = current_time
            except ImportError as e:
                logging.error(f"Error reloading: {e}")


"""SQLAlchemy models for JetTask WebUI database."""
from datetime import datetime
from typing import Optional, Dict, Any, List
from sqlalchemy import (
    Column, String, Integer, Float, DateTime, Text, JSON, 
    ARRAY, UniqueConstraint, Index, func
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.postgresql import JSONB

Base = declarative_base()


class Task(Base):
    """任务表模型"""
    __tablename__ = 'tasks'
    
    id = Column(String(255), primary_key=True)  # Redis Stream的事件ID
    queue_name = Column(String(255), nullable=False)
    task_name = Column(String(255), nullable=False)
    task_data = Column(JSONB)  # 任务的原始数据
    priority = Column(Integer, default=0)
    retry_count = Column(Integer, default=0)
    max_retry = Column(Integer, default=3)
    status = Column(String(50), default='pending')  # pending, running, success, failed, timeout
    result = Column(JSONB)  # 执行结果
    error_message = Column(Text)
    created_at = Column(DateTime(timezone=True), default=func.current_timestamp())
    started_at = Column(DateTime(timezone=True))
    completed_at = Column(DateTime(timezone=True))
    worker_id = Column(String(255))
    execution_time = Column(Float)  # 任务执行时间（秒）
    duration = Column(Float)  # 任务总持续时间（秒）
    task_metadata = Column('metadata', JSONB)  # 额外的元数据，在数据库中仍叫metadata
    next_sync_time = Column(Float, default=0)  # 下次同步时间的时间戳
    sync_check_count = Column(Integer, default=0)  # 同步检查次数
    
    __table_args__ = (
        Index('idx_tasks_queue_name', 'queue_name'),
        Index('idx_tasks_status', 'status'),
        # 分区查询索引
        Index(
            'idx_tasks_id_partition',
            func.mod(func.cast(func.split_part(id, '-', 2), Integer), 10),
            'next_sync_time', 'id', 'sync_check_count',
            postgresql_where=(status.in_(['pending', 'running']))
        ),
        # 覆盖索引
        Index(
            'idx_tasks_covering_sync',
            'next_sync_time', 'status', 'id', 'sync_check_count',
            postgresql_where=(status.in_(['pending', 'running']))
        ),
    )


class QueueStats(Base):
    """队列统计表模型"""
    __tablename__ = 'queue_stats'
    
    id = Column(Integer, primary_key=True, autoincrement=True)
    queue_name = Column(String(255), nullable=False, unique=True)
    total_tasks = Column(Integer, default=0)
    pending_tasks = Column(Integer, default=0)
    running_tasks = Column(Integer, default=0)
    completed_tasks = Column(Integer, default=0)
    failed_tasks = Column(Integer, default=0)
    avg_processing_time = Column(String)  # PostgreSQL INTERVAL type stored as string
    last_updated = Column(DateTime(timezone=True), default=func.current_timestamp())


class Worker(Base):
    """Worker信息表模型"""
    __tablename__ = 'workers'
    
    id = Column(String(255), primary_key=True)
    hostname = Column(String(255))
    process_id = Column(Integer)
    queues = Column(ARRAY(Text))  # 订阅的队列列表
    status = Column(String(50), default='active')  # active, inactive, dead
    last_heartbeat = Column(DateTime(timezone=True))
    started_at = Column(DateTime(timezone=True), default=func.current_timestamp())
    worker_metadata = Column('metadata', JSONB)  # 元数据，在数据库中仍叫metadata


"""
独立的数据访问模块，不依赖 integrated_gradio_app.py
"""
import asyncio
import json
import logging
from datetime import datetime, timedelta, timezone
from typing import Dict, List, Optional, Tuple
import redis.asyncio as redis
from sqlalchemy import text
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker

# 设置日志
logger = logging.getLogger(__name__)


class RedisConfig:
    """Redis配置"""
    def __init__(self, host='localhost', port=6379, db=0, password=None):
        self.host = host
        self.port = port
        self.db = db
        self.password = password
    
    @classmethod
    def from_env(cls):
        import os
        return cls(
            host=os.getenv('REDIS_HOST', 'localhost'),
            port=int(os.getenv('REDIS_PORT', 6379)),
            db=int(os.getenv('REDIS_DB', 0)),
            password=os.getenv('REDIS_PASSWORD')
        )


class PostgreSQLConfig:
    """PostgreSQL配置"""
    def __init__(self, host='localhost', port=5432, user='postgres', password='', database='jettask'):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
    
    @property
    def dsn(self):
        return f"postgresql://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}"
    
    @classmethod
    def from_env(cls):
        import os
        return cls(
            host=os.getenv('POSTGRES_HOST', 'localhost'),
            port=int(os.getenv('POSTGRES_PORT', 5432)),
            user=os.getenv('POSTGRES_USER', 'jettask'),
            password=os.getenv('POSTGRES_PASSWORD', '123456'),
            database=os.getenv('POSTGRES_DB', 'jettask')
        )


class JetTaskDataAccess:
    """JetTask数据访问类"""
    
    def __init__(self):
        self.redis_config = RedisConfig.from_env()
        self.pg_config = PostgreSQLConfig.from_env()
        self.redis_prefix = "jettask"
        self.async_engine = None
        self.AsyncSessionLocal = None
        self._redis_pool = None
        
    async def initialize(self):
        """初始化数据库连接"""
        try:
            # 初始化PostgreSQL引擎
            dsn = self.pg_config.dsn
            if dsn.startswith('postgresql://'):
                dsn = dsn.replace('postgresql://', 'postgresql+psycopg://', 1)
            
            self.async_engine = create_async_engine(
                dsn,
                pool_size=10,
                max_overflow=5,
                pool_pre_ping=True,
                echo=False
            )
            
            self.AsyncSessionLocal = sessionmaker(
                bind=self.async_engine,
                class_=AsyncSession,
                expire_on_commit=False
            )
            
            # 初始化Redis连接池
            self._redis_pool = redis.ConnectionPool(
                host=self.redis_config.host,
                port=self.redis_config.port,
                db=self.redis_config.db,
                password=self.redis_config.password,
                encoding='utf-8',
                decode_responses=True
            )
            
            logger.info("数据库连接初始化成功")
            
        except Exception as e:
            logger.error(f"数据库连接初始化失败: {e}")
            raise
    
    async def close(self):
        """关闭数据库连接"""
        if self.async_engine:
            await self.async_engine.dispose()
        if self._redis_pool:
            await self._redis_pool.disconnect()
    
    async def get_redis_client(self):
        """获取Redis客户端"""
        return redis.Redis(connection_pool=self._redis_pool)
    
    async def fetch_queues_data(self) -> List[Dict]:
        """获取队列数据（基于Redis Stream）"""
        try:
            redis_client = await self.get_redis_client()
            
            # 获取所有Stream类型的队列 - JetTask使用 jettask:QUEUE:队列名 格式
            all_keys = await redis_client.keys(f"{self.redis_prefix}:QUEUE:*")
            queues_data = []
            queue_names = set()
            
            for key in all_keys:
                # 检查是否是Stream类型
                key_type = await redis_client.type(key)
                if key_type == 'stream':
                    # 解析队列名称 - 格式: jettask:QUEUE:队列名
                    parts = key.split(':')
                    if len(parts) >= 3 and parts[0] == self.redis_prefix and parts[1] == 'QUEUE':
                        queue_name = ':'.join(parts[2:])  # 支持带冒号的队列名
                        queue_names.add(queue_name)
            
            # 获取每个队列的详细信息
            for queue_name in queue_names:
                stream_key = f"{self.redis_prefix}:QUEUE:{queue_name}"
                
                try:
                    # 获取Stream信息
                    stream_info = await redis_client.xinfo_stream(stream_key)
                    
                    # 获取消费者组信息
                    groups_info = await redis_client.xinfo_groups(stream_key)
                    
                    pending_count = 0
                    processing_count = 0
                    
                    # 统计各消费者组的待处理消息
                    for group in groups_info:
                        pending_count += group.get('pending', 0)
                        
                        # 获取消费者信息
                        try:
                            consumers = await redis_client.xinfo_consumers(stream_key, group['name'])
                            for consumer in consumers:
                                processing_count += consumer.get('pending', 0)
                        except:
                            pass
                    
                    # Stream的长度即为总消息数
                    total_messages = stream_info.get('length', 0)
                    
                    # 完成的消息数 = 总消息数 - 待处理 - 处理中
                    completed_count = max(0, total_messages - pending_count - processing_count)
                    
                    queues_data.append({
                        '队列名称': queue_name,
                        '待处理': pending_count,
                        '处理中': processing_count,
                        '已完成': completed_count,
                        '失败': 0,  # Stream中没有直接的失败计数
                        '总计': total_messages
                    })
                    
                except Exception as e:
                    logger.warning(f"获取队列 {queue_name} 信息失败: {e}")
                    # 如果获取详细信息失败，至少返回队列名称
                    queues_data.append({
                        '队列名称': queue_name,
                        '待处理': 0,
                        '处理中': 0,
                        '已完成': 0,
                        '失败': 0,
                        '总计': 0
                    })
            
            await redis_client.close()
            return sorted(queues_data, key=lambda x: x['队列名称'])
            
        except Exception as e:
            logger.error(f"获取队列数据失败: {e}")
            return []
    
    async def fetch_queue_timeline_data(self, 
                                      queues: List[str], 
                                      start_time: datetime, 
                                      end_time: datetime) -> List[Dict]:
        """获取队列时间线数据"""
        try:
            await asyncio.sleep(1)
            if not self.AsyncSessionLocal:
                await self.initialize()
            
            async with self.AsyncSessionLocal() as session:
                # 构建SQL查询
                queue_names_str = "', '".join(queues)
                
                # 动态计算时间间隔，目标是生成约200个时间点
                TARGET_POINTS = 200
                duration = (end_time - start_time).total_seconds()
                
                # 计算理想的间隔秒数
                ideal_interval_seconds = duration / TARGET_POINTS
                
                # 将间隔秒数规范化到合理的值
                if ideal_interval_seconds <= 1:
                    interval_seconds = 1
                    interval = '1 seconds'
                elif ideal_interval_seconds <= 5:
                    interval_seconds = 5
                    interval = '5 seconds'
                elif ideal_interval_seconds <= 10:
                    interval_seconds = 10
                    interval = '10 seconds'
                elif ideal_interval_seconds <= 30:
                    interval_seconds = 30
                    interval = '30 seconds'
                elif ideal_interval_seconds <= 60:
                    interval_seconds = 60
                    interval = '1 minute'
                elif ideal_interval_seconds <= 120:
                    interval_seconds = 120
                    interval = '2 minutes'
                elif ideal_interval_seconds <= 300:
                    interval_seconds = 300
                    interval = '5 minutes'
                elif ideal_interval_seconds <= 600:
                    interval_seconds = 600
                    interval = '10 minutes'
                elif ideal_interval_seconds <= 900:
                    interval_seconds = 900
                    interval = '15 minutes'
                elif ideal_interval_seconds <= 1800:
                    interval_seconds = 1800
                    interval = '30 minutes'
                elif ideal_interval_seconds <= 3600:
                    interval_seconds = 3600
                    interval = '1 hour'
                elif ideal_interval_seconds <= 7200:
                    interval_seconds = 7200
                    interval = '2 hours'
                elif ideal_interval_seconds <= 14400:
                    interval_seconds = 14400
                    interval = '4 hours'
                elif ideal_interval_seconds <= 21600:
                    interval_seconds = 21600
                    interval = '6 hours'
                elif ideal_interval_seconds <= 43200:
                    interval_seconds = 43200
                    interval = '12 hours'
                else:
                    interval_seconds = 86400
                    interval = '1 day'
                
                # 重新计算实际点数
                actual_points = int(duration / interval_seconds) + 1
                logger.info(f"使用时间间隔: {interval_seconds=} {interval=}, 预计生成 {actual_points} 个时间点")
                
                # 生成所有预期的时间点（根据粒度对齐）
                from datetime import timedelta
                expected_time_points = []
                current_time = start_time
                
                # 根据粒度对齐起始时间和确定截断单位
                if interval_seconds >= 86400:  # 天级别
                    current_time = current_time.replace(hour=0, minute=0, second=0, microsecond=0)
                    trunc_unit = 'day'
                elif interval_seconds >= 3600:  # 小时级别及以上
                    current_time = current_time.replace(minute=0, second=0, microsecond=0)
                    # 对于多小时间隔，对齐到间隔的整数倍
                    if interval_seconds > 3600:
                        hours_interval = interval_seconds // 3600
                        current_time = current_time.replace(
                            hour=(current_time.hour // hours_interval) * hours_interval
                        )
                    trunc_unit = 'hour'
                elif interval_seconds >= 60:  # 分钟级别
                    current_time = current_time.replace(second=0, microsecond=0)
                    # 对于多分钟间隔，对齐到间隔的整数倍
                    if interval_seconds > 60:
                        minutes_interval = interval_seconds // 60
                        current_time = current_time.replace(
                            minute=(current_time.minute // minutes_interval) * minutes_interval
                        )
                    trunc_unit = 'minute'
                else:  # 秒级别
                    current_time = current_time.replace(microsecond=0)
                    # 对于多秒间隔，对齐到间隔的整数倍
                    if interval_seconds > 1:
                        current_time = current_time.replace(
                            second=(current_time.second // interval_seconds) * interval_seconds
                        )
                    trunc_unit = 'second'
                
                # 生成时间点列表
                while current_time <= end_time:
                    expected_time_points.append(current_time)
                    current_time += timedelta(seconds=interval_seconds)
                
                logger.info(f"生成了 {len(expected_time_points)} 个时间点")
                
                # 简化的SQL查询 - 只做基本的时间聚合
                query = text(f"""
                    SELECT 
                        date_trunc('{trunc_unit}', 
                            :start_time + INTERVAL '{interval}' * 
                            FLOOR(EXTRACT(epoch FROM created_at - :start_time) / {interval_seconds})
                        ) as time_bucket,
                        queue_name,
                        COUNT(*) as task_count
                    FROM tasks 
                    WHERE queue_name IN ('{queue_names_str}')
                        AND created_at >= :start_time 
                        AND created_at <= :end_time
                    GROUP BY time_bucket, queue_name
                    ORDER BY time_bucket, queue_name
                """)
                
                result = await session.execute(query, {
                    'start_time': start_time,
                    'end_time': end_time
                })
                
                # 将查询结果存储在字典中，方便查找
                data_dict = {}
                for row in result:
                    # 根据粒度对齐时间
                    time_bucket = row.time_bucket
                    if interval_seconds >= 86400:  # 天级别
                        time_bucket = time_bucket.replace(hour=0, minute=0, second=0, microsecond=0)
                    elif interval_seconds >= 3600:  # 小时级别及以上
                        time_bucket = time_bucket.replace(minute=0, second=0, microsecond=0)
                        if interval_seconds > 3600:
                            hours_interval = interval_seconds // 3600
                            time_bucket = time_bucket.replace(
                                hour=(time_bucket.hour // hours_interval) * hours_interval
                            )
                    elif interval_seconds >= 60:  # 分钟级别
                        time_bucket = time_bucket.replace(second=0, microsecond=0)
                        if interval_seconds > 60:
                            minutes_interval = interval_seconds // 60
                            time_bucket = time_bucket.replace(
                                minute=(time_bucket.minute // minutes_interval) * minutes_interval
                            )
                    else:  # 秒级别
                        time_bucket = time_bucket.replace(microsecond=0)
                        if interval_seconds > 1:
                            time_bucket = time_bucket.replace(
                                second=(time_bucket.second // interval_seconds) * interval_seconds
                            )
                    
                    time_key = time_bucket.isoformat()
                    if time_key not in data_dict:
                        data_dict[time_key] = {}
                    data_dict[time_key][row.queue_name] = row.task_count
                
                logger.info(f"从DB查询到 {len(data_dict)} 个时间点的数据")
                
                # 生成完整的时间线数据，包括填充缺失的时间点
                timeline_data = []
                for time_point in expected_time_points:
                    time_str = time_point.isoformat()
                    for queue in queues:
                        # 检查该时间点是否有数据，没有则用0填充
                        value = 0
                        if time_str in data_dict and queue in data_dict[time_str]:
                            value = data_dict[time_str][queue]
                        
                        timeline_data.append({
                            'time': time_str,
                            'queue': queue,
                            'value': value
                        })
                
                return timeline_data
                
        except Exception as e:
            logger.error(f"获取队列时间线数据失败: {e}")
            return []
    
    
    async def fetch_global_stats(self) -> Dict:
        """获取全局统计信息"""
        try:
            redis_client = await self.get_redis_client()
            
            # 获取所有队列的统计信息
            queues_data = await self.fetch_queues_data()
            
            total_pending = sum(q['待处理'] for q in queues_data)
            total_processing = sum(q['处理中'] for q in queues_data)
            total_completed = sum(q['已完成'] for q in queues_data)
            total_failed = sum(q['失败'] for q in queues_data)
            
            # 获取活跃worker数量
            worker_keys = await redis_client.keys(f"{self.redis_prefix}:worker:*")
            active_workers = len(worker_keys)
            
            await redis_client.close()
            
            return {
                'total_queues': len(queues_data),
                'total_pending': total_pending,
                'total_processing': total_processing,
                'total_completed': total_completed,
                'total_failed': total_failed,
                'active_workers': active_workers,
                'timestamp': datetime.now(timezone.utc).isoformat()
            }
            
        except Exception as e:
            logger.error(f"获取全局统计信息失败: {e}")
            return {}